# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..dwi import FitDwi


def test_FitDwi_inputs():
    input_map = dict(
        acceptance=dict(argstr='-accpetance %f', ),
        args=dict(argstr='%s', ),
        ball_flag=dict(
            argstr='-ball',
            position=4,
            xor=[
                'mono_flag', 'ivim_flag', 'dti_flag', 'ballv_flag', 'nod_flag',
                'nodv_flag'
            ],
        ),
        ballv_flag=dict(
            argstr='-ballv',
            position=4,
            xor=[
                'mono_flag', 'ivim_flag', 'dti_flag', 'ball_flag', 'nod_flag',
                'nodv_flag'
            ],
        ),
        bval_file=dict(
            argstr='-bval %s',
            extensions=None,
            mandatory=True,
            position=2,
        ),
        bvec_file=dict(
            argstr='-bvec %s',
            extensions=None,
            mandatory=True,
            position=3,
        ),
        cov_file=dict(
            argstr='-cov %s',
            extensions=None,
        ),
        csf_t2_val=dict(argstr='-csfT2 %f', ),
        diso_val=dict(argstr='-diso %f', ),
        dpr_val=dict(argstr='-dpr %f', ),
        dti_flag=dict(
            argstr='-dti',
            position=4,
            xor=[
                'mono_flag', 'ivim_flag', 'ball_flag', 'ballv_flag',
                'nod_flag', 'nodv_flag'
            ],
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        error_file=dict(
            argstr='-error %s',
            extensions=None,
            name_source=['source_file'],
            name_template='%s_error.nii.gz',
        ),
        famap_file=dict(
            argstr='-famap %s',
            extensions=None,
            name_source=['source_file'],
            name_template='%s_famap.nii.gz',
        ),
        gn_flag=dict(
            argstr='-gn',
            xor=['wls_flag'],
        ),
        ivim_flag=dict(
            argstr='-ivim',
            position=4,
            xor=[
                'mono_flag', 'dti_flag', 'ball_flag', 'ballv_flag', 'nod_flag',
                'nodv_flag'
            ],
        ),
        lm_vals=dict(
            argstr='-lm %f %f',
            requires=['gn_flag'],
        ),
        mask_file=dict(
            argstr='-mask %s',
            extensions=None,
        ),
        maxit_val=dict(
            argstr='-maxit %d',
            requires=['gn_flag'],
        ),
        mcmap_file=dict(
            argstr='-mcmap %s',
            extensions=None,
            name_source=['source_file'],
            name_template='%s_mcmap.nii.gz',
            requires=['nodv_flag'],
        ),
        mcmaxit=dict(argstr='-mcmaxit %d', ),
        mcout=dict(
            argstr='-mcout %s',
            extensions=None,
            name_source=['source_file'],
            name_template='%s_mcout.txt',
        ),
        mcsamples=dict(argstr='-mcsamples %d', ),
        mdmap_file=dict(
            argstr='-mdmap %s',
            extensions=None,
            name_source=['source_file'],
            name_template='%s_mdmap.nii.gz',
        ),
        mono_flag=dict(
            argstr='-mono',
            position=4,
            xor=[
                'ivim_flag', 'dti_flag', 'ball_flag', 'ballv_flag', 'nod_flag',
                'nodv_flag'
            ],
        ),
        nod_flag=dict(
            argstr='-nod',
            position=4,
            xor=[
                'mono_flag', 'ivim_flag', 'dti_flag', 'ball_flag',
                'ballv_flag', 'nodv_flag'
            ],
        ),
        nodiff_file=dict(
            argstr='-nodiff %s',
            extensions=None,
            name_source=['source_file'],
            name_template='%s_no_diff.nii.gz',
        ),
        nodv_flag=dict(
            argstr='-nodv',
            position=4,
            xor=[
                'mono_flag', 'ivim_flag', 'dti_flag', 'ball_flag',
                'ballv_flag', 'nod_flag'
            ],
        ),
        perf_thr=dict(argstr='-perfthreshold %f', ),
        prior_file=dict(
            argstr='-prior %s',
            extensions=None,
        ),
        res_file=dict(
            argstr='-res %s',
            extensions=None,
            name_source=['source_file'],
            name_template='%s_resmap.nii.gz',
        ),
        rgbmap_file=dict(
            argstr='-rgbmap %s',
            extensions=None,
            name_source=['source_file'],
            name_template='%s_rgbmap.nii.gz',
            requires=['dti_flag'],
        ),
        rot_sform_flag=dict(argstr='-rotsform %d', ),
        slice_no=dict(argstr='-slice %d', ),
        source_file=dict(
            argstr='-source %s',
            extensions=None,
            mandatory=True,
            position=1,
        ),
        swls_val=dict(argstr='-swls %f', ),
        syn_file=dict(
            argstr='-syn %s',
            extensions=None,
            name_source=['source_file'],
            name_template='%s_syn.nii.gz',
        ),
        te_file=dict(
            argstr='-TE %s',
            extensions=None,
            xor=['te_file'],
        ),
        te_value=dict(
            argstr='-TE %s',
            extensions=None,
            xor=['te_file'],
        ),
        ten_type=dict(usedefault=True, ),
        tenmap2_file=dict(
            argstr='-tenmap2 %s',
            extensions=None,
            name_source=['source_file'],
            name_template='%s_tenmap2.nii.gz',
            requires=['dti_flag'],
        ),
        tenmap_file=dict(
            argstr='-tenmap %s',
            extensions=None,
            name_source=['source_file'],
            name_template='%s_tenmap.nii.gz',
            requires=['dti_flag'],
        ),
        v1map_file=dict(
            argstr='-v1map %s',
            extensions=None,
            name_source=['source_file'],
            name_template='%s_v1map.nii.gz',
        ),
        vb_flag=dict(argstr='-vb', ),
        voxel=dict(argstr='-voxel %d %d %d', ),
        wls_flag=dict(
            argstr='-wls',
            xor=['gn_flag'],
        ),
        wm_t2_val=dict(argstr='-wmT2 %f', ),
    )
    inputs = FitDwi.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_FitDwi_outputs():
    output_map = dict(
        error_file=dict(extensions=None, ),
        famap_file=dict(extensions=None, ),
        mcmap_file=dict(extensions=None, ),
        mcout=dict(extensions=None, ),
        mdmap_file=dict(extensions=None, ),
        nodiff_file=dict(extensions=None, ),
        res_file=dict(extensions=None, ),
        rgbmap_file=dict(extensions=None, ),
        syn_file=dict(extensions=None, ),
        tenmap2_file=dict(extensions=None, ),
        tenmap_file=dict(extensions=None, ),
        v1map_file=dict(extensions=None, ),
    )
    outputs = FitDwi.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..specialized import BRAINSCut


def test_BRAINSCut_inputs():
    input_map = dict(
        NoTrainingVectorShuffling=dict(
            argstr='--NoTrainingVectorShuffling ', ),
        applyModel=dict(argstr='--applyModel ', ),
        args=dict(argstr='%s', ),
        computeSSEOn=dict(argstr='--computeSSEOn ', ),
        createVectors=dict(argstr='--createVectors ', ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        generateProbability=dict(argstr='--generateProbability ', ),
        histogramEqualization=dict(argstr='--histogramEqualization ', ),
        method=dict(argstr='--method %s', ),
        modelConfigurationFilename=dict(
            argstr='--modelConfigurationFilename %s',
            extensions=None,
        ),
        modelFilename=dict(argstr='--modelFilename %s', ),
        multiStructureThreshold=dict(argstr='--multiStructureThreshold ', ),
        netConfiguration=dict(
            argstr='--netConfiguration %s',
            extensions=None,
        ),
        numberOfTrees=dict(argstr='--numberOfTrees %d', ),
        randomTreeDepth=dict(argstr='--randomTreeDepth %d', ),
        trainModel=dict(argstr='--trainModel ', ),
        trainModelStartIndex=dict(argstr='--trainModelStartIndex %d', ),
        validate=dict(argstr='--validate ', ),
        verbose=dict(argstr='--verbose %d', ),
    )
    inputs = BRAINSCut.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_BRAINSCut_outputs():
    output_map = dict()
    outputs = BRAINSCut.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..model import Label2Vol


def test_Label2Vol_inputs():
    input_map = dict(
        annot_file=dict(
            argstr='--annot %s',
            copyfile=False,
            extensions=None,
            mandatory=True,
            requires=('subject_id', 'hemi'),
            xor=('label_file', 'annot_file', 'seg_file', 'aparc_aseg'),
        ),
        aparc_aseg=dict(
            argstr='--aparc+aseg',
            mandatory=True,
            xor=('label_file', 'annot_file', 'seg_file', 'aparc_aseg'),
        ),
        args=dict(argstr='%s', ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        fill_thresh=dict(argstr='--fillthresh %g', ),
        hemi=dict(argstr='--hemi %s', ),
        identity=dict(
            argstr='--identity',
            xor=('reg_file', 'reg_header', 'identity'),
        ),
        invert_mtx=dict(argstr='--invertmtx', ),
        label_file=dict(
            argstr='--label %s...',
            copyfile=False,
            mandatory=True,
            xor=('label_file', 'annot_file', 'seg_file', 'aparc_aseg'),
        ),
        label_hit_file=dict(
            argstr='--hits %s',
            extensions=None,
        ),
        label_voxel_volume=dict(argstr='--labvoxvol %f', ),
        map_label_stat=dict(
            argstr='--label-stat %s',
            extensions=None,
        ),
        native_vox2ras=dict(argstr='--native-vox2ras', ),
        proj=dict(
            argstr='--proj %s %f %f %f',
            requires=('subject_id', 'hemi'),
        ),
        reg_file=dict(
            argstr='--reg %s',
            extensions=None,
            xor=('reg_file', 'reg_header', 'identity'),
        ),
        reg_header=dict(
            argstr='--regheader %s',
            extensions=None,
            xor=('reg_file', 'reg_header', 'identity'),
        ),
        seg_file=dict(
            argstr='--seg %s',
            copyfile=False,
            extensions=None,
            mandatory=True,
            xor=('label_file', 'annot_file', 'seg_file', 'aparc_aseg'),
        ),
        subject_id=dict(argstr='--subject %s', ),
        subjects_dir=dict(),
        surface=dict(argstr='--surf %s', ),
        template_file=dict(
            argstr='--temp %s',
            extensions=None,
            mandatory=True,
        ),
        vol_label_file=dict(
            argstr='--o %s',
            extensions=None,
            genfile=True,
        ),
    )
    inputs = Label2Vol.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_Label2Vol_outputs():
    output_map = dict(vol_label_file=dict(extensions=None, ), )
    outputs = Label2Vol.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..preprocess import TSmooth


def test_TSmooth_inputs():
    input_map = dict(
        adaptive=dict(argstr='-adaptive %d', ),
        args=dict(argstr='%s', ),
        blackman=dict(argstr='-blackman %d', ),
        custom=dict(
            argstr='-custom %s',
            extensions=None,
        ),
        datum=dict(argstr='-datum %s', ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        hamming=dict(argstr='-hamming %d', ),
        in_file=dict(
            argstr='%s',
            copyfile=False,
            extensions=None,
            mandatory=True,
            position=-1,
        ),
        lin=dict(argstr='-lin', ),
        lin3=dict(argstr='-3lin %d', ),
        med=dict(argstr='-med', ),
        num_threads=dict(
            nohash=True,
            usedefault=True,
        ),
        osf=dict(argstr='-osf', ),
        out_file=dict(
            argstr='-prefix %s',
            extensions=None,
            name_source='in_file',
            name_template='%s_smooth',
        ),
        outputtype=dict(),
    )
    inputs = TSmooth.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_TSmooth_outputs():
    output_map = dict(out_file=dict(extensions=None, ), )
    outputs = TSmooth.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..dti import ProbTrackX


def test_ProbTrackX_inputs():
    input_map = dict(
        args=dict(argstr='%s', ),
        avoid_mp=dict(
            argstr='--avoid=%s',
            extensions=None,
        ),
        c_thresh=dict(argstr='--cthr=%.3f', ),
        correct_path_distribution=dict(argstr='--pd', ),
        dist_thresh=dict(argstr='--distthresh=%.3f', ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        fibst=dict(argstr='--fibst=%d', ),
        force_dir=dict(
            argstr='--forcedir',
            usedefault=True,
        ),
        fsamples=dict(mandatory=True, ),
        inv_xfm=dict(
            argstr='--invxfm=%s',
            extensions=None,
        ),
        loop_check=dict(argstr='--loopcheck', ),
        mask=dict(
            argstr='-m %s',
            extensions=None,
            mandatory=True,
        ),
        mask2=dict(
            argstr='--mask2=%s',
            extensions=None,
        ),
        mesh=dict(
            argstr='--mesh=%s',
            extensions=None,
        ),
        mod_euler=dict(argstr='--modeuler', ),
        mode=dict(
            argstr='--mode=%s',
            genfile=True,
        ),
        n_samples=dict(
            argstr='--nsamples=%d',
            usedefault=True,
        ),
        n_steps=dict(argstr='--nsteps=%d', ),
        network=dict(argstr='--network', ),
        opd=dict(
            argstr='--opd',
            usedefault=True,
        ),
        os2t=dict(argstr='--os2t', ),
        out_dir=dict(
            argstr='--dir=%s',
            genfile=True,
        ),
        output_type=dict(),
        phsamples=dict(mandatory=True, ),
        rand_fib=dict(argstr='--randfib=%d', ),
        random_seed=dict(argstr='--rseed', ),
        s2tastext=dict(argstr='--s2tastext', ),
        sample_random_points=dict(argstr='--sampvox', ),
        samples_base_name=dict(
            argstr='--samples=%s',
            usedefault=True,
        ),
        seed=dict(
            argstr='--seed=%s',
            mandatory=True,
        ),
        seed_ref=dict(
            argstr='--seedref=%s',
            extensions=None,
        ),
        step_length=dict(argstr='--steplength=%.3f', ),
        stop_mask=dict(
            argstr='--stop=%s',
            extensions=None,
        ),
        target_masks=dict(argstr='--targetmasks=%s', ),
        thsamples=dict(mandatory=True, ),
        use_anisotropy=dict(argstr='--usef', ),
        verbose=dict(argstr='--verbose=%d', ),
        waypoints=dict(
            argstr='--waypoints=%s',
            extensions=None,
        ),
        xfm=dict(
            argstr='--xfm=%s',
            extensions=None,
        ),
    )
    inputs = ProbTrackX.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_ProbTrackX_outputs():
    output_map = dict(
        fdt_paths=dict(),
        log=dict(extensions=None, ),
        particle_files=dict(),
        targets=dict(),
        way_total=dict(extensions=None, ),
    )
    outputs = ProbTrackX.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..minc import Extract


def test_Extract_inputs():
    input_map = dict(
        args=dict(argstr='%s', ),
        count=dict(
            argstr='-count %s',
            sep=',',
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        flip_any_direction=dict(
            argstr='-any_direction',
            xor=('flip_positive_direction', 'flip_negative_direction',
                 'flip_any_direction'),
        ),
        flip_negative_direction=dict(
            argstr='-negative_direction',
            xor=('flip_positive_direction', 'flip_negative_direction',
                 'flip_any_direction'),
        ),
        flip_positive_direction=dict(
            argstr='-positive_direction',
            xor=('flip_positive_direction', 'flip_negative_direction',
                 'flip_any_direction'),
        ),
        flip_x_any=dict(
            argstr='-xanydirection',
            xor=('flip_x_positive', 'flip_x_negative', 'flip_x_any'),
        ),
        flip_x_negative=dict(
            argstr='-xdirection',
            xor=('flip_x_positive', 'flip_x_negative', 'flip_x_any'),
        ),
        flip_x_positive=dict(
            argstr='+xdirection',
            xor=('flip_x_positive', 'flip_x_negative', 'flip_x_any'),
        ),
        flip_y_any=dict(
            argstr='-yanydirection',
            xor=('flip_y_positive', 'flip_y_negative', 'flip_y_any'),
        ),
        flip_y_negative=dict(
            argstr='-ydirection',
            xor=('flip_y_positive', 'flip_y_negative', 'flip_y_any'),
        ),
        flip_y_positive=dict(
            argstr='+ydirection',
            xor=('flip_y_positive', 'flip_y_negative', 'flip_y_any'),
        ),
        flip_z_any=dict(
            argstr='-zanydirection',
            xor=('flip_z_positive', 'flip_z_negative', 'flip_z_any'),
        ),
        flip_z_negative=dict(
            argstr='-zdirection',
            xor=('flip_z_positive', 'flip_z_negative', 'flip_z_any'),
        ),
        flip_z_positive=dict(
            argstr='+zdirection',
            xor=('flip_z_positive', 'flip_z_negative', 'flip_z_any'),
        ),
        image_maximum=dict(argstr='-image_maximum %s', ),
        image_minimum=dict(argstr='-image_minimum %s', ),
        image_range=dict(argstr='-image_range %s %s', ),
        input_file=dict(
            argstr='%s',
            extensions=None,
            mandatory=True,
            position=-2,
        ),
        nonormalize=dict(
            argstr='-nonormalize',
            xor=('normalize', 'nonormalize'),
        ),
        normalize=dict(
            argstr='-normalize',
            xor=('normalize', 'nonormalize'),
        ),
        out_file=dict(
            argstr='> %s',
            extensions=None,
            genfile=True,
            position=-1,
        ),
        output_file=dict(
            extensions=None,
            hash_files=False,
            keep_extension=False,
            name_source=['input_file'],
            name_template='%s.raw',
            position=-1,
        ),
        start=dict(
            argstr='-start %s',
            sep=',',
        ),
        write_ascii=dict(
            argstr='-ascii',
            xor=('write_ascii', 'write_ascii', 'write_byte', 'write_short',
                 'write_int', 'write_long', 'write_float', 'write_double',
                 'write_signed', 'write_unsigned'),
        ),
        write_byte=dict(
            argstr='-byte',
            xor=('write_ascii', 'write_ascii', 'write_byte', 'write_short',
                 'write_int', 'write_long', 'write_float', 'write_double',
                 'write_signed', 'write_unsigned'),
        ),
        write_double=dict(
            argstr='-double',
            xor=('write_ascii', 'write_ascii', 'write_byte', 'write_short',
                 'write_int', 'write_long', 'write_float', 'write_double',
                 'write_signed', 'write_unsigned'),
        ),
        write_float=dict(
            argstr='-float',
            xor=('write_ascii', 'write_ascii', 'write_byte', 'write_short',
                 'write_int', 'write_long', 'write_float', 'write_double',
                 'write_signed', 'write_unsigned'),
        ),
        write_int=dict(
            argstr='-int',
            xor=('write_ascii', 'write_ascii', 'write_byte', 'write_short',
                 'write_int', 'write_long', 'write_float', 'write_double',
                 'write_signed', 'write_unsigned'),
        ),
        write_long=dict(
            argstr='-long',
            xor=('write_ascii', 'write_ascii', 'write_byte', 'write_short',
                 'write_int', 'write_long', 'write_float', 'write_double',
                 'write_signed', 'write_unsigned'),
        ),
        write_range=dict(argstr='-range %s %s', ),
        write_short=dict(
            argstr='-short',
            xor=('write_ascii', 'write_ascii', 'write_byte', 'write_short',
                 'write_int', 'write_long', 'write_float', 'write_double',
                 'write_signed', 'write_unsigned'),
        ),
        write_signed=dict(
            argstr='-signed',
            xor=('write_signed', 'write_unsigned'),
        ),
        write_unsigned=dict(
            argstr='-unsigned',
            xor=('write_signed', 'write_unsigned'),
        ),
    )
    inputs = Extract.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_Extract_outputs():
    output_map = dict(output_file=dict(extensions=None, ), )
    outputs = Extract.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# -*- coding: utf-8 -*-
"""Parallel workflow execution via Condor
"""
from __future__ import (print_function, division, unicode_literals,
                        absolute_import)

import os
from time import sleep

from ...interfaces.base import CommandLine
from ... import logging
from .base import SGELikeBatchManagerBase, logger
iflogger = logging.getLogger('nipype.interface')


class CondorPlugin(SGELikeBatchManagerBase):
    """Execute using Condor

    This plugin doesn't work with a plain stock-Condor installation, but
    requires a 'qsub' emulation script for Condor, called 'condor_qsub'.
    This script is shipped with the Condor package from NeuroDebian, or can be
    downloaded from its Git repository at

    http://anonscm.debian.org/gitweb/?p=pkg-exppsy/condor.git;a=blob_plain;f=debian/condor_qsub;hb=HEAD

    The plugin_args input to run can be used to control the Condor execution.
    Currently supported options are:

    - template : template to use for batch job submission. This can be an
                 SGE-style script with the (limited) set of options supported
                 by condor_qsub
    - qsub_args : arguments to be prepended to the job execution script in the
                  qsub call
    """

    def __init__(self, **kwargs):
        template = """
#$ -V
#$ -S /bin/sh
        """
        self._retry_timeout = 2
        self._max_tries = 2
        if 'plugin_args' in kwargs and kwargs['plugin_args']:
            if 'retry_timeout' in kwargs['plugin_args']:
                self._retry_timeout = kwargs['plugin_args']['retry_timeout']
            if 'max_tries' in kwargs['plugin_args']:
                self._max_tries = kwargs['plugin_args']['max_tries']
        super(CondorPlugin, self).__init__(template, **kwargs)

    def _is_pending(self, taskid):
        cmd = CommandLine(
            'condor_q', resource_monitor=False, terminal_output='allatonce')
        cmd.inputs.args = '%d' % taskid
        # check condor cluster
        oldlevel = iflogger.level
        iflogger.setLevel(logging.getLevelName('CRITICAL'))
        result = cmd.run(ignore_exception=True)
        iflogger.setLevel(oldlevel)
        if result.runtime.stdout.count('\n%d' % taskid):
            return True
        return False

    def _submit_batchtask(self, scriptfile, node):
        cmd = CommandLine(
            'condor_qsub',
            environ=dict(os.environ),
            resource_monitor=False,
            terminal_output='allatonce')
        path = os.path.dirname(scriptfile)
        qsubargs = ''
        if self._qsub_args:
            qsubargs = self._qsub_args
        if 'qsub_args' in node.plugin_args:
            if 'overwrite' in node.plugin_args and\
               node.plugin_args['overwrite']:
                qsubargs = node.plugin_args['qsub_args']
            else:
                qsubargs += (" " + node.plugin_args['qsub_args'])
        if self._qsub_args:
            qsubargs = self._qsub_args
        if '-o' not in qsubargs:
            qsubargs = '%s -o %s' % (qsubargs, path)
        if '-e' not in qsubargs:
            qsubargs = '%s -e %s' % (qsubargs, path)
        if node._hierarchy:
            jobname = '.'.join((dict(os.environ)['LOGNAME'], node._hierarchy,
                                node._id))
        else:
            jobname = '.'.join((dict(os.environ)['LOGNAME'], node._id))
        jobnameitems = jobname.split('.')
        jobnameitems.reverse()
        jobname = '.'.join(jobnameitems)
        cmd.inputs.args = '%s -N %s %s' % (qsubargs, jobname, scriptfile)
        oldlevel = iflogger.level
        iflogger.setLevel(logging.getLevelName('CRITICAL'))
        tries = 0
        while True:
            try:
                result = cmd.run()
            except Exception as e:
                if tries < self._max_tries:
                    tries += 1
                    sleep(self._retry_timeout)  # sleep 2 seconds and try again
                else:
                    iflogger.setLevel(oldlevel)
                    raise RuntimeError('\n'.join((('Could not submit condor '
                                                   'cluster'
                                                   ' for node %s') % node._id,
                                                  str(e))))
            else:
                break
        iflogger.setLevel(oldlevel)
        # retrieve condor clusterid
        taskid = int(result.runtime.stdout.split(' ')[2])
        self._pending[taskid] = node.output_dir()
        logger.debug('submitted condor cluster: %d for node %s' % (taskid,
                                                                   node._id))
        return taskid



# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
import os

import nibabel as nb
import numpy as np

import pytest
from ...testing import utils
from ..confounds import CompCor, TCompCor, ACompCor


class TestCompCor():
    ''' Note: Tests currently do a poor job of testing functionality '''

    filenames = {
        'functionalnii': 'compcorfunc.nii',
        'masknii': 'compcormask.nii',
        'masknii2': 'compcormask2.nii',
        'components_file': None
    }

    @pytest.fixture(autouse=True)
    def setup_class(self, tmpdir):
        # setup
        tmpdir.chdir()
        noise = np.fromfunction(self.fake_noise_fun, self.fake_data.shape)
        self.realigned_file = utils.save_toy_nii(
            self.fake_data + noise, self.filenames['functionalnii'])
        mask = np.ones(self.fake_data.shape[:3])
        mask[0, 0, 0] = 0
        mask[0, 0, 1] = 0
        mask1 = utils.save_toy_nii(mask, self.filenames['masknii'])

        other_mask = np.ones(self.fake_data.shape[:3])
        other_mask[0, 1, 0] = 0
        other_mask[1, 1, 0] = 0
        mask2 = utils.save_toy_nii(other_mask, self.filenames['masknii2'])

        self.mask_files = [mask1, mask2]

    def test_compcor(self):
        expected_components = [['-0.1989607212', '-0.5753813646'], [
            '0.5692369697', '0.5674945949'
        ], ['-0.6662573243',
            '0.4675843432'], ['0.4206466244', '-0.3361270124'],
                               ['-0.1246655485', '-0.1235705610']]

        self.run_cc(
            CompCor(
                num_components=6,
                realigned_file=self.realigned_file,
                mask_files=self.mask_files,
                mask_index=0), expected_components)

        self.run_cc(
            ACompCor(
                num_components=6,
                realigned_file=self.realigned_file,
                mask_files=self.mask_files,
                mask_index=0,
                components_file='acc_components_file'), expected_components,
            'aCompCor')

    def test_compcor_variance_threshold_and_metadata(self):
        expected_components = [['-0.2027150345', '-0.4954813834'],
                               ['0.2565929051', '0.7866217875'],
                               ['-0.3550986008', '-0.0089784905'],
                               ['0.7512786244', '-0.3599828482'],
                               ['-0.4500578942', '0.0778209345']]
        expected_metadata = {
            'component': 'CompCor00',
            'mask': 'mask',
            'singular_value': '4.0720553036',
            'variance_explained': '0.5527211465',
            'cumulative_variance_explained': '0.5527211465',
            'retained': 'True',
        }
        ccinterface = CompCor(
                variance_threshold=0.7,
                realigned_file=self.realigned_file,
                mask_files=self.mask_files,
                mask_names=['mask'],
                mask_index=1,
                save_metadata=True)
        self.run_cc(ccinterface=ccinterface,
                    expected_components=expected_components,
                    expected_n_components=2,
                    expected_metadata=expected_metadata)

    def test_tcompcor(self):
        ccinterface = TCompCor(num_components=6,
            realigned_file=self.realigned_file, percentile_threshold=0.75)
        self.run_cc(ccinterface, [['-0.1114536190', '-0.4632908609'], [
            '0.4566907310', '0.6983205193'
        ], ['-0.7132557407', '0.1340170559'], [
            '0.5022537643', '-0.5098322262'
        ], ['-0.1342351356', '0.1407855119']], 'tCompCor')

    def test_tcompcor_no_percentile(self):
        ccinterface = TCompCor(num_components=6,
            realigned_file=self.realigned_file)
        ccinterface.run()

        mask = nb.load('mask_000.nii.gz').get_data()
        num_nonmasked_voxels = np.count_nonzero(mask)
        assert num_nonmasked_voxels == 1

    def test_compcor_no_regress_poly(self):
        self.run_cc(
            CompCor(
                num_components=6,
                realigned_file=self.realigned_file,
                mask_files=self.mask_files,
                mask_index=0,
                pre_filter=False), [['0.4451946442', '-0.7683311482'], [
                    '-0.4285129505', '-0.0926034137'
                ], ['0.5721540256', '0.5608764842'], [
                    '-0.5367548139', '0.0059943226'
                ], ['-0.0520809054', '0.2940637551']])

    def test_tcompcor_asymmetric_dim(self):
        asymmetric_shape = (2, 3, 4, 5)
        asymmetric_data = utils.save_toy_nii(
            np.zeros(asymmetric_shape), 'asymmetric.nii')

        TCompCor(realigned_file=asymmetric_data).run()
        assert nb.load(
            'mask_000.nii.gz').get_data().shape == asymmetric_shape[:3]

    def test_compcor_bad_input_shapes(self):
        # dim 0 is < dim 0 of self.mask_files (2)
        shape_less_than = (1, 2, 2, 5)
        # dim 0 is > dim 0 of self.mask_files (2)
        shape_more_than = (3, 3, 3, 5)

        for data_shape in (shape_less_than, shape_more_than):
            data_file = utils.save_toy_nii(np.zeros(data_shape), 'temp.nii')
            interface = CompCor(
                realigned_file=data_file, mask_files=self.mask_files[0])
            with pytest.raises(ValueError):
                interface.run()  # Dimension mismatch

    def test_tcompcor_bad_input_dim(self):
        bad_dims = (2, 2, 2)
        data_file = utils.save_toy_nii(np.zeros(bad_dims), 'temp.nii')
        interface = TCompCor(realigned_file=data_file)
        with pytest.raises(ValueError):
            interface.run()  # Not a 4D file

    def test_tcompcor_merge_intersect_masks(self):
        for method in ['union', 'intersect']:
            TCompCor(
                realigned_file=self.realigned_file,
                mask_files=self.mask_files,
                merge_method=method).run()
            if method == 'union':
                assert np.array_equal(
                    nb.load('mask_000.nii.gz').get_data(),
                    ([[[0, 0], [0, 0]], [[0, 0], [1, 0]]]))
            if method == 'intersect':
                assert np.array_equal(
                    nb.load('mask_000.nii.gz').get_data(),
                    ([[[0, 0], [0, 0]], [[0, 1], [0, 0]]]))

    def test_tcompcor_index_mask(self):
        TCompCor(
            realigned_file=self.realigned_file,
            mask_files=self.mask_files,
            mask_index=1).run()
        assert np.array_equal(
            nb.load('mask_000.nii.gz').get_data(),
            ([[[0, 0], [0, 0]], [[0, 1], [0, 0]]]))

    def test_tcompcor_multi_mask_no_index(self):
        interface = TCompCor(
            realigned_file=self.realigned_file, mask_files=self.mask_files)
        with pytest.raises(ValueError):
            interface.run()  # more than one mask file

    def run_cc(self,
               ccinterface,
               expected_components,
               expected_header='CompCor',
               expected_n_components=None,
               expected_metadata=None):
        # run
        ccresult = ccinterface.run()

        # assert
        expected_file = ccinterface._list_outputs()['components_file']
        assert ccresult.outputs.components_file == expected_file
        assert os.path.exists(expected_file)
        assert os.path.getsize(expected_file) > 0

        with open(ccresult.outputs.components_file, 'r') as components_file:
            if expected_n_components is None:
                expected_n_components = min(ccinterface.inputs.num_components,
                                            self.fake_data.shape[3])

            components_data = [line.rstrip().split('\t')
                               for line in components_file]

            # the first item will be '#', we can throw it out
            header = components_data.pop(0)
            expected_header = [
                expected_header + '{:02d}'.format(i)
                for i in range(expected_n_components)
            ]
            for i, heading in enumerate(header):
                assert expected_header[i] in heading

            num_got_timepoints = len(components_data)
            assert num_got_timepoints == self.fake_data.shape[3]
            for index, timepoint in enumerate(components_data):
                assert (len(timepoint) == expected_n_components)
                assert timepoint[:2] == expected_components[index]

        if ccinterface.inputs.save_metadata:
            expected_metadata_file = (
                ccinterface._list_outputs()['metadata_file'])
            assert ccresult.outputs.metadata_file == expected_metadata_file
            assert os.path.exists(expected_metadata_file)
            assert os.path.getsize(expected_metadata_file) > 0

            with open(ccresult.outputs.metadata_file, 'r') as metadata_file:
                components_metadata = [line.rstrip().split('\t')
                                       for line in metadata_file]
                components_metadata = {i: j for i, j in
                                        zip(components_metadata[0],
                                            components_metadata[1])}
                assert components_metadata == expected_metadata

        return ccresult

    @staticmethod
    def fake_noise_fun(i, j, l, m):
        return m * i + l - j

    fake_data = np.array([[[[8, 5, 3, 8, 0], [6, 7, 4, 7, 1]],
                           [[7, 9, 1, 6, 5], [0, 7, 4, 7, 7]]],
                          [[[2, 4, 5, 7, 0], [1, 7, 0, 5, 4]],
                           [[7, 3, 9, 0, 4], [9, 4, 1, 5, 0]]]])



# -*- coding: utf-8 -*-
from __future__ import absolute_import
from .specialized import (RobustStatisticsSegmenter, EMSegmentCommandLine,
                          BRAINSROIAuto)
from .simpleregiongrowingsegmentation import SimpleRegionGrowingSegmentation



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..brainsuite import Skullfinder


def test_Skullfinder_inputs():
    input_map = dict(
        args=dict(argstr='%s', ),
        bgLabelValue=dict(argstr='--bglabel %d', ),
        brainLabelValue=dict(argstr='--brainlabel %d', ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        inputMRIFile=dict(
            argstr='-i %s',
            extensions=None,
            mandatory=True,
        ),
        inputMaskFile=dict(
            argstr='-m %s',
            extensions=None,
            mandatory=True,
        ),
        lowerThreshold=dict(argstr='-l %d', ),
        outputLabelFile=dict(
            argstr='-o %s',
            extensions=None,
            genfile=True,
        ),
        performFinalOpening=dict(argstr='--finalOpening', ),
        scalpLabelValue=dict(argstr='--scalplabel %d', ),
        skullLabelValue=dict(argstr='--skulllabel %d', ),
        spaceLabelValue=dict(argstr='--spacelabel %d', ),
        surfaceFilePrefix=dict(argstr='-s %s', ),
        upperThreshold=dict(argstr='-u %d', ),
        verbosity=dict(argstr='-v %d', ),
    )
    inputs = Skullfinder.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_Skullfinder_outputs():
    output_map = dict(outputLabelFile=dict(extensions=None, ), )
    outputs = Skullfinder.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..tensors import FSL2MRTrix


def test_FSL2MRTrix_inputs():
    input_map = dict(
        bval_file=dict(
            extensions=None,
            mandatory=True,
        ),
        bvec_file=dict(
            extensions=None,
            mandatory=True,
        ),
        invert_x=dict(usedefault=True, ),
        invert_y=dict(usedefault=True, ),
        invert_z=dict(usedefault=True, ),
        out_encoding_file=dict(
            extensions=None,
            genfile=True,
        ),
    )
    inputs = FSL2MRTrix.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_FSL2MRTrix_outputs():
    output_map = dict(encoding_file=dict(extensions=None, ), )
    outputs = FSL2MRTrix.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..fiberprocess import fiberprocess


def test_fiberprocess_inputs():
    input_map = dict(
        args=dict(argstr='%s', ),
        displacement_field=dict(
            argstr='--displacement_field %s',
            extensions=None,
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        fiber_file=dict(
            argstr='--fiber_file %s',
            extensions=None,
        ),
        fiber_output=dict(
            argstr='--fiber_output %s',
            hash_files=False,
        ),
        fiber_radius=dict(argstr='--fiber_radius %f', ),
        h_field=dict(
            argstr='--h_field %s',
            extensions=None,
        ),
        index_space=dict(argstr='--index_space ', ),
        noDataChange=dict(argstr='--noDataChange ', ),
        no_warp=dict(argstr='--no_warp ', ),
        saveProperties=dict(argstr='--saveProperties ', ),
        tensor_volume=dict(
            argstr='--tensor_volume %s',
            extensions=None,
        ),
        verbose=dict(argstr='--verbose ', ),
        voxel_label=dict(argstr='--voxel_label %d', ),
        voxelize=dict(
            argstr='--voxelize %s',
            hash_files=False,
        ),
        voxelize_count_fibers=dict(argstr='--voxelize_count_fibers ', ),
    )
    inputs = fiberprocess.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_fiberprocess_outputs():
    output_map = dict(
        fiber_output=dict(extensions=None, ),
        voxelize=dict(extensions=None, ),
    )
    outputs = fiberprocess.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
"""
The possum module provides classes for interfacing with `POSSUM
<http://fsl.fmrib.ox.ac.uk/fsl/fslwiki/POSSUM>`_ command line tools.
Please, check out the link for pertinent citations using POSSUM.

  .. Note:: This was written to work with FSL version 5.0.6.
"""

from .base import FSLCommand, FSLCommandInputSpec
from ..base import TraitedSpec, File, traits


class B0CalcInputSpec(FSLCommandInputSpec):
    in_file = File(
        exists=True,
        mandatory=True,
        argstr='-i %s',
        position=0,
        desc='filename of input image (usually a tissue/air segmentation)')
    out_file = File(
        argstr='-o %s',
        position=1,
        name_source=['in_file'],
        name_template='%s_b0field',
        output_name='out_file',
        desc='filename of B0 output volume')

    x_grad = traits.Float(
        0.0, usedefault=True,
        argstr='--gx=%0.4f',
        desc='Value for zeroth-order x-gradient field (per mm)')
    y_grad = traits.Float(
        0.0, usedefault=True,
        argstr='--gy=%0.4f',
        desc='Value for zeroth-order y-gradient field (per mm)')
    z_grad = traits.Float(
        0.0, usedefault=True,
        argstr='--gz=%0.4f',
        desc='Value for zeroth-order z-gradient field (per mm)')

    x_b0 = traits.Float(
        0.0, usedefault=True,
        argstr='--b0x=%0.2f',
        xor=['xyz_b0'],
        desc='Value for zeroth-order b0 field (x-component), in Tesla')
    y_b0 = traits.Float(
        0.0, usedefault=True,
        argstr='--b0y=%0.2f',
        xor=['xyz_b0'],
        desc='Value for zeroth-order b0 field (y-component), in Tesla')
    z_b0 = traits.Float(
        1.0, usedefault=True,
        argstr='--b0=%0.2f',
        xor=['xyz_b0'],
        desc='Value for zeroth-order b0 field (z-component), in Tesla')

    xyz_b0 = traits.Tuple(
        traits.Float,
        traits.Float,
        traits.Float,
        argstr='--b0x=%0.2f --b0y=%0.2f --b0=%0.2f',
        xor=['x_b0', 'y_b0', 'z_b0'],
        desc='Zeroth-order B0 field in Tesla')

    delta = traits.Float(
        -9.45e-6, usedefault=True,
         argstr='-d %e', desc='Delta value (chi_tissue - chi_air)')
    chi_air = traits.Float(
        4.0e-7, usedefault=True,
        argstr='--chi0=%e', desc='susceptibility of air')
    compute_xyz = traits.Bool(
        False, usedefault=True,
        argstr='--xyz',
        desc='calculate and save all 3 field components (i.e. x,y,z)')
    extendboundary = traits.Float(
        1.0, usedefault=True,
        argstr='--extendboundary=%0.2f',
        desc='Relative proportion to extend voxels at boundary')
    directconv = traits.Bool(
        False, usedefault=True,
        argstr='--directconv',
        desc='use direct (image space) convolution, not FFT')


class B0CalcOutputSpec(TraitedSpec):
    out_file = File(exists=True, desc='filename of B0 output volume')


class B0Calc(FSLCommand):
    """
    B0 inhomogeneities occur at interfaces of materials with different magnetic susceptibilities,
    such as tissue-air interfaces. These differences lead to distortion in the local magnetic field,
    as Maxwell’s equations need to be satisfied. An example of B0 inhomogneity is the first volume
    of the 4D volume ```$FSLDIR/data/possum/b0_ppm.nii.gz```.

    Examples
    --------

    >>> from nipype.interfaces.fsl import B0Calc
    >>> b0calc = B0Calc()
    >>> b0calc.inputs.in_file = 'tissue+air_map.nii'
    >>> b0calc.inputs.z_b0 = 3.0
    >>> b0calc.inputs.output_type = "NIFTI_GZ"
    >>> b0calc.cmdline
    'b0calc -i tissue+air_map.nii -o tissue+air_map_b0field.nii.gz --chi0=4.000000e-07 \
-d -9.450000e-06 --extendboundary=1.00 --b0x=0.00 --gx=0.0000 --b0y=0.00 --gy=0.0000 \
--b0=3.00 --gz=0.0000'

    """

    _cmd = 'b0calc'
    input_spec = B0CalcInputSpec
    output_spec = B0CalcOutputSpec



# -*- coding: utf-8 -*-
# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
from __future__ import print_function, unicode_literals
from builtins import str
import os
from copy import deepcopy
import pytest

from .... import config
from ....interfaces import utility as niu
from ... import engine as pe
from ..utils import merge_dict
from .test_base import EngineTestInterface
from .test_utils import UtilsTestInterface

'''
Test for order of iterables

import nipype.pipeline.engine as pe
import nipype.interfaces.utility as niu

wf1 = pe.Workflow(name='wf1')
node1 = pe.Node(interface=niu.IdentityInterface(fields=['a1','b1']), name='node1')
node1.iterables = ('a1', [1,2])
wf1.add_nodes([node1])

wf2 = pe.Workflow(name='wf2')
node2 = pe.Node(interface=niu.IdentityInterface(fields=['a2','b2']), name='node2')
wf2.add_nodes([node2])
wf1.connect(node1, 'a1', wf2, 'node2.a2')

node4 = pe.Node(interface=niu.IdentityInterface(fields=['a4','b4']), name='node4')
#node4.iterables = ('a4', [5,6])
wf2.connect(node2, 'b2', node4, 'b4')

wf3 = pe.Workflow(name='wf3')
node3 = pe.Node(interface=niu.IdentityInterface(fields=['a3','b3']), name='node3')
node3.iterables = ('b3', [3,4])
wf3.add_nodes([node3])
wf1.connect(wf3, 'node3.b3', wf2, 'node2.b2')

wf1.base_dir = os.path.join(os.getcwd(),'testit')
wf1.run(inseries=True, createdirsonly=True)

wf1.write_graph(graph2use='exec')
'''
'''
import nipype.pipeline.engine as pe
import nipype.interfaces.spm as spm
import os
from io import StringIO
from nipype.utils.config import config

config.readfp(StringIO("""
[execution]
remove_unnecessary_outputs = true
"""))


segment = pe.Node(interface=spm.Segment(), name="segment")
segment.inputs.data = os.path.abspath("data/T1.nii")
segment.inputs.gm_output_type = [True, True, True]
segment.inputs.wm_output_type = [True, True, True]


smooth_gm = pe.Node(interface=spm.Smooth(), name="smooth_gm")

workflow = pe.Workflow(name="workflow_cleanup_test")
workflow.base_dir = os.path.abspath('./workflow_cleanup_test')

workflow.connect([(segment, smooth_gm, [('native_gm_image','in_files')])])

workflow.run()

#adding new node that uses one of the previously deleted outputs of segment; this should force segment to rerun
smooth_wm = pe.Node(interface=spm.Smooth(), name="smooth_wm")

workflow.connect([(segment, smooth_wm, [('native_wm_image','in_files')])])

workflow.run()

workflow.run()
'''

# Node


def test_node_init():
    with pytest.raises(TypeError):
        pe.Node()
    with pytest.raises(IOError):
        pe.Node(EngineTestInterface, name='test')


def test_node_get_output():
    mod1 = pe.Node(interface=EngineTestInterface(), name='mod1')
    mod1.inputs.input1 = 1
    mod1.run()
    assert mod1.get_output('output1') == [1, 1]
    mod1._result = None
    assert mod1.get_output('output1') == [1, 1]


def test_mapnode_iterfield_check():
    mod1 = pe.MapNode(EngineTestInterface(), iterfield=['input1'], name='mod1')
    with pytest.raises(ValueError):
        mod1._check_iterfield()
    mod1 = pe.MapNode(
        EngineTestInterface(), iterfield=['input1', 'input2'], name='mod1')
    mod1.inputs.input1 = [1, 2]
    mod1.inputs.input2 = 3
    with pytest.raises(ValueError):
        mod1._check_iterfield()


@pytest.mark.parametrize("x_inp, f_exp",
                         [(3, [6]), ([2, 3], [4, 6]), ((2, 3), [4, 6]),
                          (range(3), [0, 2, 4]), ("Str", ["StrStr"]),
                          (["Str1", "Str2"], ["Str1Str1", "Str2Str2"])])
def test_mapnode_iterfield_type(x_inp, f_exp):
    from nipype import MapNode, Function

    def double_func(x):
        return 2 * x

    double = Function(["x"], ["f_x"], double_func)

    double_node = MapNode(double, name="double", iterfield=["x"])
    double_node.inputs.x = x_inp

    res = double_node.run()
    assert res.outputs.f_x == f_exp


def test_mapnode_nested(tmpdir):
    tmpdir.chdir()
    from nipype import MapNode, Function

    def func1(in1):
        return in1 + 1

    n1 = MapNode(
        Function(input_names=['in1'], output_names=['out'], function=func1),
        iterfield=['in1'],
        nested=True,
        name='n1')
    n1.inputs.in1 = [[1, [2]], 3, [4, 5]]
    n1.run()
    assert n1.get_output('out') == [[2, [3]], 4, [5, 6]]

    n2 = MapNode(
        Function(input_names=['in1'], output_names=['out'], function=func1),
        iterfield=['in1'],
        nested=False,
        name='n1')
    n2.inputs.in1 = [[1, [2]], 3, [4, 5]]

    with pytest.raises(Exception) as excinfo:
        n2.run()
    assert "can only concatenate list" in str(excinfo.value)


def test_mapnode_expansion(tmpdir):
    tmpdir.chdir()
    from nipype import MapNode, Function

    def func1(in1):
        return in1 + 1

    mapnode = MapNode(
        Function(function=func1),
        iterfield='in1',
        name='mapnode',
        n_procs=2,
        mem_gb=2)
    mapnode.inputs.in1 = [1, 2]

    for idx, node in mapnode._make_nodes():
        for attr in ('overwrite', 'run_without_submitting', 'plugin_args'):
            assert getattr(node, attr) == getattr(mapnode, attr)
        for attr in ('_n_procs', '_mem_gb'):
            assert (getattr(node, attr) == getattr(mapnode, attr))


def test_node_hash(tmpdir):
    from nipype.interfaces.utility import Function
    tmpdir.chdir()

    config.set_default_config()
    config.set('execution', 'stop_on_first_crash', True)
    config.set('execution', 'crashdump_dir', os.getcwd())

    def func1():
        return 1

    def func2(a):
        return a + 1

    n1 = pe.Node(
        Function(input_names=[], output_names=['a'], function=func1),
        name='n1')
    n2 = pe.Node(
        Function(input_names=['a'], output_names=['b'], function=func2),
        name='n2')
    w1 = pe.Workflow(name='test')

    def modify(x):
        return x + 1
    n1.inputs.a = 1
    w1.connect(n1, ('a', modify), n2, 'a')
    w1.base_dir = os.getcwd()

    # create dummy distributed plugin class
    from nipype.pipeline.plugins.base import DistributedPluginBase

    # create a custom exception
    class EngineTestException(Exception):
        pass

    class RaiseError(DistributedPluginBase):
        def _submit_job(self, node, updatehash=False):
            raise EngineTestException(
                'Submit called - cached=%s, updated=%s' % node.is_cached())

    # check if a proper exception is raised
    with pytest.raises(EngineTestException) as excinfo:
        w1.run(plugin=RaiseError())
    assert str(excinfo.value).startswith('Submit called')

    # generate outputs
    w1.run(plugin='Linear')
    # ensure plugin is being called
    config.set('execution', 'local_hash_check', False)

    # rerun to ensure we have outputs
    w1.run(plugin='Linear')

    # set local check
    config.set('execution', 'local_hash_check', True)
    w1 = pe.Workflow(name='test')
    w1.connect(n1, ('a', modify), n2, 'a')
    w1.base_dir = os.getcwd()
    w1.run(plugin=RaiseError())


def test_outputs_removal(tmpdir):
    def test_function(arg1):
        import os
        file1 = os.path.join(os.getcwd(), 'file1.txt')
        file2 = os.path.join(os.getcwd(), 'file2.txt')
        with open(file1, 'wt') as fp:
            fp.write('%d' % arg1)
        with open(file2, 'wt') as fp:
            fp.write('%d' % arg1)
        return file1, file2

    n1 = pe.Node(
        niu.Function(
            input_names=['arg1'],
            output_names=['file1', 'file2'],
            function=test_function),
        base_dir=tmpdir.strpath,
        name='testoutputs')
    n1.inputs.arg1 = 1
    n1.config = {'execution': {'remove_unnecessary_outputs': True}}
    n1.config = merge_dict(deepcopy(config._sections), n1.config)
    n1.run()
    assert tmpdir.join(n1.name, 'file1.txt').check()
    assert tmpdir.join(n1.name, 'file1.txt').check()
    n1.needed_outputs = ['file2']
    n1.run()
    assert not tmpdir.join(n1.name, 'file1.txt').check()
    assert tmpdir.join(n1.name, 'file2.txt').check()


def test_inputs_removal(tmpdir):
    file1 = tmpdir.join('file1.txt')
    file1.write('dummy_file')
    n1 = pe.Node(
        UtilsTestInterface(), base_dir=tmpdir.strpath, name='testinputs')
    n1.inputs.in_file = file1.strpath
    n1.config = {'execution': {'keep_inputs': True}}
    n1.config = merge_dict(deepcopy(config._sections), n1.config)
    n1.run()
    assert tmpdir.join(n1.name, 'file1.txt').check()
    n1.inputs.in_file = file1.strpath
    n1.config = {'execution': {'keep_inputs': False}}
    n1.config = merge_dict(deepcopy(config._sections), n1.config)
    n1.overwrite = True
    n1.run()
    assert not tmpdir.join(n1.name, 'file1.txt').check()


def test_outputmultipath_collapse(tmpdir):
    """Test an OutputMultiPath whose initial value is ``[[x]]`` to ensure that
    it is returned as ``[x]``, regardless of how accessed."""
    select_if = niu.Select(inlist=[[1, 2, 3], [4]], index=1)
    select_nd = pe.Node(niu.Select(inlist=[[1, 2, 3], [4]], index=1),
                        name='select_nd')

    ifres = select_if.run()
    ndres = select_nd.run()

    assert ifres.outputs.out == [4]
    assert ndres.outputs.out == [4]
    assert select_nd.result.outputs.out == [4]



# -*- coding: utf-8 -*-
"""The ants module provides basic functions for interfacing with ants
   functions.
"""
from __future__ import (print_function, division, unicode_literals,
                        absolute_import)
import os
from glob import glob

from .base import (CommandLineInputSpec, traits, TraitedSpec, File,
                   SEMLikeCommandLine, InputMultiPath, OutputMultiPath,
                   CommandLine, isdefined)
from ..utils.filemanip import split_filename
from .. import logging

iflogger = logging.getLogger("interface")


class C3dAffineToolInputSpec(CommandLineInputSpec):
    reference_file = File(exists=True, argstr="-ref %s", position=1)
    source_file = File(exists=True, argstr='-src %s', position=2)
    transform_file = File(exists=True, argstr='%s', position=3)
    itk_transform = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc="Export ITK transform.",
        argstr="-oitk %s",
        position=5)
    fsl2ras = traits.Bool(argstr='-fsl2ras', position=4)


class C3dAffineToolOutputSpec(TraitedSpec):
    itk_transform = File(exists=True)


class C3dAffineTool(SEMLikeCommandLine):
    """Converts fsl-style Affine registration into ANTS compatible itk format

    Example
    =======

    >>> from nipype.interfaces.c3 import C3dAffineTool
    >>> c3 = C3dAffineTool()
    >>> c3.inputs.source_file = 'cmatrix.mat'
    >>> c3.inputs.itk_transform = 'affine.txt'
    >>> c3.inputs.fsl2ras = True
    >>> c3.cmdline
    'c3d_affine_tool -src cmatrix.mat -fsl2ras -oitk affine.txt'
    """
    input_spec = C3dAffineToolInputSpec
    output_spec = C3dAffineToolOutputSpec

    _cmd = 'c3d_affine_tool'
    _outputs_filenames = {'itk_transform': 'affine.txt'}


class C3dInputSpec(CommandLineInputSpec):
    in_file = InputMultiPath(
        File(),
        position=1,
        argstr="%s",
        mandatory=True,
        desc="Input file (wildcard and multiple are supported).")
    out_file = File(
        exists=False,
        argstr="-o %s",
        position=-1,
        xor=["out_files"],
        desc="Output file of last image on the stack.")
    out_files = InputMultiPath(
        File(),
        argstr="-oo %s",
        xor=["out_file"],
        position=-1,
        desc=("Write all images on the convert3d stack as multiple files."
        " Supports both list of output files or a pattern for the output"
        " filenames (using %d substituion)."))
    pix_type = traits.Enum(
        "float", "char", "uchar", "short", "ushort", "int", "uint", "double",
        argstr="-type %s",
        desc=("Specifies the pixel type for the output image. By default,"
        " images are written in floating point (float) format"))
    scale = traits.Either(
        traits.Int(), traits.Float(),
        argstr="-scale %s",
        desc=("Multiplies the intensity of each voxel in the last image on the"
        " stack by the given factor."))
    shift = traits.Either(
        traits.Int(), traits.Float(),
        argstr="-shift %s",
        desc='Adds the given constant to every voxel.')
    interp = traits.Enum(
        "Linear", "NearestNeighbor", "Cubic", "Sinc", "Gaussian",
        argstr="-interpolation %s",
        desc=("Specifies the interpolation used with -resample and other"
        " commands. Default is Linear."))
    resample = traits.Str(
        argstr="-resample %s",
        desc=("Resamples the image, keeping the bounding box the same, but"
        " changing the number of voxels in the image. The dimensions can be"
        " specified as a percentage, for example to double the number of voxels"
        " in each direction. The -interpolation flag affects how sampling is"
        " performed."))
    smooth = traits.Str(
        argstr="-smooth %s",
        desc=("Applies Gaussian smoothing to the image. The parameter vector"
        " specifies the standard deviation of the Gaussian kernel."))
    multicomp_split = traits.Bool(
        False,
        usedefault=True,
        argstr="-mcr",
        position=0,
        desc="Enable reading of multi-component images.")
    is_4d = traits.Bool(
        False,
        usedefault=True,
        desc=("Changes command to support 4D file operations (default is"
        " false)."))


class C3dOutputSpec(TraitedSpec):
    out_files = OutputMultiPath(File(exists=False))


class C3d(CommandLine):
    """
    Convert3d is a command-line tool for converting 3D (or 4D) images between
    common file formats. The tool also includes a growing list of commands for
    image manipulation, such as thresholding and resampling. The tool can also
    be used to obtain information about image files. More information on
    Convert3d can be found at:
    https://sourceforge.net/p/c3d/git/ci/master/tree/doc/c3d.md


    Example
    =======

    >>> from nipype.interfaces.c3 import C3d
    >>> c3 = C3d()
    >>> c3.inputs.in_file = "T1.nii"
    >>> c3.inputs.pix_type = "short"
    >>> c3.inputs.out_file = "T1.img"
    >>> c3.cmdline
    'c3d T1.nii -type short -o T1.img'
    >>> c3.inputs.is_4d = True
    >>> c3.inputs.in_file = "epi.nii"
    >>> c3.inputs.out_file = "epi.img"
    >>> c3.cmdline
    'c4d epi.nii -type short -o epi.img'
    """
    input_spec = C3dInputSpec
    output_spec = C3dOutputSpec

    _cmd = "c3d"

    def __init__(self, **inputs):
        super(C3d, self).__init__(**inputs)
        self.inputs.on_trait_change(self._is_4d, "is_4d")
        if self.inputs.is_4d:
            self._is_4d()

    def _is_4d(self):
        self._cmd = "c4d" if self.inputs.is_4d else "c3d"

    def _run_interface(self, runtime):
        cmd = self._cmd
        if (not isdefined(self.inputs.out_file)
            and not isdefined(self.inputs.out_files)):
            # Convert3d does not want to override file, by default
            # so we define a new output file
            self._gen_outfile()
        runtime = super(C3d, self)._run_interface(runtime)
        self._cmd = cmd
        return runtime

    def _gen_outfile(self):
        # if many infiles, raise exception
        if (len(self.inputs.in_file) > 1) or ("*" in self.inputs.in_file[0]):
            raise AttributeError("Multiple in_files found - specify either"
                                 " `out_file` or `out_files`.")
        _, fn, ext = split_filename(self.inputs.in_file[0])
        self.inputs.out_file = fn + "_generated" + ext
        # if generated file will overwrite, raise error
        if os.path.exists(os.path.abspath(self.inputs.out_file)):
            raise IOError("File already found - to overwrite, use `out_file`.")
        iflogger.info("Generating `out_file`.")

    def _list_outputs(self):
        outputs = self.output_spec().get()
        if isdefined(self.inputs.out_file):
            outputs["out_files"] = os.path.abspath(self.inputs.out_file)
        if isdefined(self.inputs.out_files):
            if len(self.inputs.out_files) == 1:
                _out_files = glob(os.path.abspath(self.inputs.out_files[0]))
            else:
                _out_files = [os.path.abspath(f) for f in self.inputs.out_files
                              if os.path.exists(os.path.abspath(f))]
            outputs["out_files"] = _out_files

        return outputs




# -*- coding: utf-8 -*-
# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
"""Camino top level namespace
"""

from .connectivity import Conmat
from .convert import (Image2Voxel, FSL2Scheme, VtkStreamlines, ProcStreamlines,
                      TractShredder, DT2NIfTI, NIfTIDT2Camino, AnalyzeHeader,
                      Shredder)
from .dti import (DTIFit, ModelFit, DTLUTGen, PicoPDFs, Track, TrackPICo,
                  TrackBayesDirac, TrackDT, TrackBallStick, TrackBootstrap,
                  TrackBedpostxDeter, TrackBedpostxProba,
                  ComputeFractionalAnisotropy, ComputeMeanDiffusivity,
                  ComputeTensorTrace, ComputeEigensystem, DTMetric)
from .calib import (SFPICOCalibData, SFLUTGen)
from .odf import (QBallMX, LinRecon, SFPeaks, MESD)
from .utils import ImageStats



# -*- coding: utf-8 -*-
from __future__ import absolute_import
from .specialized import (VBRAINSDemonWarp, BRAINSDemonWarp,
                          BRAINSTransformFromFiducials)
from .brainsresample import BRAINSResample
from .brainsfit import BRAINSFit
from .brainsresize import BRAINSResize



# -*- coding: utf-8 -*-
"""The ants visualisation module provides basic functions based on ITK.
"""
from __future__ import (print_function, division, unicode_literals,
                        absolute_import)

import os

from ..base import TraitedSpec, File, traits
from .base import ANTSCommand, ANTSCommandInputSpec


class ConvertScalarImageToRGBInputSpec(ANTSCommandInputSpec):
    dimension = traits.Enum(
        3,
        2,
        argstr='%d',
        usedefault=True,
        desc='image dimension (2 or 3)',
        mandatory=True,
        position=0)
    input_image = File(
        argstr='%s',
        exists=True,
        desc='Main input is a 3-D grayscale image.',
        mandatory=True,
        position=1)
    output_image = traits.Str(
        'rgb.nii.gz',
        argstr='%s',
        usedefault=True,
        desc='rgb output image',
        position=2)
    mask_image = File(
        'none',
        argstr='%s',
        exists=True,
        desc='mask image',
        position=3,
        usedefault=True)
    colormap = traits.Str(
        argstr='%s',
        usedefault=True,
        desc=('Possible colormaps: grey, red, green, '
              'blue, copper, jet, hsv, spring, summer, '
              'autumn, winter, hot, cool, overunder, custom '),
        mandatory=True,
        position=4)
    custom_color_map_file = traits.Str(
        'none',
        argstr='%s',
        usedefault=True,
        desc='custom color map file',
        position=5)
    minimum_input = traits.Int(
        argstr='%d', desc='minimum input', mandatory=True, position=6)
    maximum_input = traits.Int(
        argstr='%d', desc='maximum input', mandatory=True, position=7)
    minimum_RGB_output = traits.Int(
        0, usedefault=True, argstr='%d', desc='', position=8)
    maximum_RGB_output = traits.Int(
        255, usedefault=True, argstr='%d', desc='', position=9)


class ConvertScalarImageToRGBOutputSpec(TraitedSpec):
    output_image = File(exists=True, desc='converted RGB image')


class ConvertScalarImageToRGB(ANTSCommand):
    """
    Examples
    --------
    >>> from nipype.interfaces.ants.visualization import ConvertScalarImageToRGB
    >>> converter = ConvertScalarImageToRGB()
    >>> converter.inputs.dimension = 3
    >>> converter.inputs.input_image = 'T1.nii.gz'
    >>> converter.inputs.colormap = 'jet'
    >>> converter.inputs.minimum_input = 0
    >>> converter.inputs.maximum_input = 6
    >>> converter.cmdline
    'ConvertScalarImageToRGB 3 T1.nii.gz rgb.nii.gz none jet none 0 6 0 255'
    """
    _cmd = 'ConvertScalarImageToRGB'
    input_spec = ConvertScalarImageToRGBInputSpec
    output_spec = ConvertScalarImageToRGBOutputSpec

    def _format_arg(self, opt, spec, val):
        return super(ConvertScalarImageToRGB, self)._format_arg(opt, spec, val)

    def _list_outputs(self):
        outputs = self._outputs().get()
        outputs['output_image'] = os.path.join(os.getcwd(),
                                               self.inputs.output_image)
        return outputs


class CreateTiledMosaicInputSpec(ANTSCommandInputSpec):
    input_image = File(
        argstr='-i %s',
        exists=True,
        desc='Main input is a 3-D grayscale image.',
        mandatory=True)
    rgb_image = File(
        argstr='-r %s',
        exists=True,
        desc=('An optional Rgb image can be added as an overlay.'
              'It must have the same image'
              'geometry as the input grayscale image.'),
        mandatory=True)
    mask_image = File(
        argstr='-x %s',
        exists=True,
        desc='Specifies the ROI of the RGB voxels used.')
    alpha_value = traits.Float(
        argstr='-a %.2f',
        desc=('If an Rgb image is provided, render the overlay '
              'using the specified alpha parameter.'))
    output_image = traits.Str(
        'output.png',
        argstr='-o %s',
        desc='The output consists of the tiled mosaic image.',
        usedefault=True)
    tile_geometry = traits.Str(
        argstr='-t %s',
        desc=(
            'The tile geometry specifies the number of rows and columns'
            'in the output image. For example, if the user specifies "5x10", '
            'then 5 rows by 10 columns of slices are rendered. If R < 0 and C > '
            '0 (or vice versa), the negative value is selected'
            'based on direction.'))
    direction = traits.Int(
        argstr='-d %d',
        desc=('Specifies the direction of '
              'the slices. If no direction is specified, the '
              'direction with the coarsest spacing is chosen.'))
    pad_or_crop = traits.Str(
        argstr='-p %s',
        desc='argument passed to -p flag:'
        '[padVoxelWidth,<constantValue=0>]'
        '[lowerPadding[0]xlowerPadding[1],upperPadding[0]xupperPadding[1],'
        'constantValue]'
        'The user can specify whether to pad or crop a specified '
        'voxel-width boundary of each individual slice. For this '
        'program, cropping is simply padding with negative voxel-widths.'
        'If one pads (+), the user can also specify a constant pad '
        'value (default = 0). If a mask is specified, the user can use '
        'the mask to define the region, by using the keyword "mask"'
        ' plus an offset, e.g. "-p mask+3".')
    slices = traits.Str(
        argstr='-s %s',
        desc=('Number of slices to increment Slice1xSlice2xSlice3'
              '[numberOfSlicesToIncrement,<minSlice=0>,<maxSlice=lastSlice>]'))
    flip_slice = traits.Str(argstr='-f %s', desc='flipXxflipY')
    permute_axes = traits.Bool(argstr='-g', desc='doPermute')


class CreateTiledMosaicOutputSpec(TraitedSpec):
    output_image = File(exists=True, desc='image file')


class CreateTiledMosaic(ANTSCommand):
    """The program CreateTiledMosaic in conjunction with ConvertScalarImageToRGB
    provides useful functionality for common image analysis tasks. The basic
    usage of CreateTiledMosaic is to tile a 3-D image volume slice-wise into
    a 2-D image.

    Examples
    --------

    >>> from nipype.interfaces.ants.visualization import CreateTiledMosaic
    >>> mosaic_slicer = CreateTiledMosaic()
    >>> mosaic_slicer.inputs.input_image = 'T1.nii.gz'
    >>> mosaic_slicer.inputs.rgb_image = 'rgb.nii.gz'
    >>> mosaic_slicer.inputs.mask_image = 'mask.nii.gz'
    >>> mosaic_slicer.inputs.output_image = 'output.png'
    >>> mosaic_slicer.inputs.alpha_value = 0.5
    >>> mosaic_slicer.inputs.direction = 2
    >>> mosaic_slicer.inputs.pad_or_crop = '[ -15x -50 , -15x -30 ,0]'
    >>> mosaic_slicer.inputs.slices = '[2 ,100 ,160]'
    >>> mosaic_slicer.cmdline
    'CreateTiledMosaic -a 0.50 -d 2 -i T1.nii.gz -x mask.nii.gz -o output.png -p [ -15x -50 , -15x -30 ,0] \
-r rgb.nii.gz -s [2 ,100 ,160]'
    """

    _cmd = 'CreateTiledMosaic'
    input_spec = CreateTiledMosaicInputSpec
    output_spec = CreateTiledMosaicOutputSpec

    def _list_outputs(self):
        outputs = self._outputs().get()
        outputs['output_image'] = os.path.join(os.getcwd(),
                                               self.inputs.output_image)
        return outputs



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..bru2nii import Bru2


def test_Bru2_inputs():
    input_map = dict(
        actual_size=dict(argstr='-a', ),
        append_protocol_name=dict(argstr='-p', ),
        args=dict(argstr='%s', ),
        compress=dict(argstr='-z', ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        force_conversion=dict(argstr='-f', ),
        input_dir=dict(
            argstr='%s',
            mandatory=True,
            position=-1,
        ),
        output_filename=dict(
            argstr='-o %s',
            genfile=True,
        ),
    )
    inputs = Bru2.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_Bru2_outputs():
    output_map = dict(nii_file=dict(extensions=None, ), )
    outputs = Bru2.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# -*- coding: utf-8 -*-
# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft = python sts = 4 ts = 4 sw = 4 et:
"""
AFNI modeling interfaces.

Examples
--------
See the docstrings of the individual classes for examples.

"""
from __future__ import (print_function, division, unicode_literals,
                        absolute_import)

import os

from ..base import (CommandLineInputSpec, CommandLine, Directory, TraitedSpec,
                    traits, isdefined, File, InputMultiPath, Undefined, Str)
from ...external.due import BibTeX

from .base import (AFNICommandBase, AFNICommand, AFNICommandInputSpec,
                   AFNICommandOutputSpec, Info)


class DeconvolveInputSpec(AFNICommandInputSpec):
    in_files = InputMultiPath(
        File(exists=True),
        desc='filenames of 3D+time input datasets. More than one filename can '
        'be given and the datasets will be auto-catenated in time. '
        'You can input a 1D time series file here, but the time axis '
        'should run along the ROW direction, not the COLUMN direction as '
        'in the \'input1D\' option.',
        argstr='-input %s',
        copyfile=False,
        sep=" ",
        position=1)
    sat = traits.Bool(
        desc='check the dataset time series for initial saturation transients,'
        ' which should normally have been excised before data analysis.',
        argstr='-sat',
        xor=['trans'])
    trans = traits.Bool(
        desc='check the dataset time series for initial saturation transients,'
        ' which should normally have been excised before data analysis.',
        argstr='-trans',
        xor=['sat'])
    noblock = traits.Bool(
        desc='normally, if you input multiple datasets with \'input\', then '
        'the separate datasets are taken to be separate image runs that '
        'get separate baseline models. Use this options if you want to '
        'have the program consider these to be all one big run.'
        '* If any of the input dataset has only 1 sub-brick, then this '
        'option is automatically invoked!'
        '* If the auto-catenation feature isn\'t used, then this option '
        'has no effect, no how, no way.',
        argstr='-noblock')
    force_TR = traits.Float(
        desc='use this value instead of the TR in the \'input\' '
             'dataset. (It\'s better to fix the input using Refit.)',
        argstr='-force_TR %f',
        position=0)
    input1D = File(
        desc='filename of single (fMRI) .1D time series where time runs down '
        'the column.',
        argstr='-input1D %s',
        exists=True)
    TR_1D = traits.Float(
        desc='TR to use with \'input1D\'. This option has no effect if you do '
        'not also use \'input1D\'.',
        argstr='-TR_1D %f')
    legendre = traits.Bool(
        desc='use Legendre polynomials for null hypothesis (baseline model)',
        argstr='-legendre')
    nolegendre = traits.Bool(
        desc='use power polynomials for null hypotheses. Don\'t do this '
        'unless you are crazy!',
        argstr='-nolegendre')
    nodmbase = traits.Bool(
        desc='don\'t de-mean baseline time series', argstr='-nodmbase')
    dmbase = traits.Bool(
        desc='de-mean baseline time series (default if \'polort\' >= 0)',
        argstr='-dmbase')
    svd = traits.Bool(
        desc='use SVD instead of Gaussian elimination (default)',
        argstr='-svd')
    nosvd = traits.Bool(
        desc='use Gaussian elimination instead of SVD', argstr='-nosvd')
    rmsmin = traits.Float(
        desc='minimum rms error to reject reduced model (default = 0; don\'t '
        'use this option normally!)',
        argstr='-rmsmin %f')
    nocond = traits.Bool(
        desc='DON\'T calculate matrix condition number', argstr='-nocond')
    singvals = traits.Bool(
        desc='print out the matrix singular values', argstr='-singvals')
    goforit = traits.Int(
        desc='use this to proceed even if the matrix has bad problems (e.g., '
        'duplicate columns, large condition number, etc.).',
        argstr='-GOFORIT %i')
    allzero_OK = traits.Bool(
        desc='don\'t consider all zero matrix columns to be the type of error '
        'that \'gotforit\' is needed to ignore.',
        argstr='-allzero_OK')
    dname = traits.Tuple(
        Str,
        Str,
        desc='set environmental variable to provided value',
        argstr='-D%s=%s')
    mask = File(
        desc='filename of 3D mask dataset; only data time series from within '
        'the mask will be analyzed; results for voxels outside the mask '
        'will be set to zero.',
        argstr='-mask %s',
        exists=True)
    automask = traits.Bool(
        desc='build a mask automatically from input data (will be slow for '
        'long time series datasets)',
        argstr='-automask')
    STATmask = File(
        desc='build a mask from provided file, and use this mask for the '
        'purpose of reporting truncation-to float issues AND for '
        'computing the FDR curves. The actual results ARE not masked '
        'with this option (only with \'mask\' or \'automask\' options).',
        argstr='-STATmask %s',
        exists=True)
    censor = File(
        desc='filename of censor .1D time series. This is a file of 1s and '
        '0s, indicating which time points are to be included (1) and '
        'which are to be excluded (0).',
        argstr='-censor %s',
        exists=True)
    polort = traits.Int(
        desc='degree of polynomial corresponding to the null hypothesis '
        '[default: 1]',
        argstr='-polort %d')
    ortvec = traits.Tuple(
        File(desc='filename', exists=True),
        Str(desc='label'),
        desc='this option lets you input a rectangular array of 1 or more '
        'baseline vectors from a file. This method is a fast way to '
        'include a lot of baseline regressors in one step. ',
        argstr='-ortvec %s %s')
    x1D = File(desc='specify name for saved X matrix', argstr='-x1D %s')
    x1D_stop = traits.Bool(
        desc='stop running after writing .xmat.1D file', argstr='-x1D_stop')
    cbucket = traits.Str(
        desc='Name for dataset in which to save the regression '
        'coefficients (no statistics). This dataset '
        'will be used in a -xrestore run [not yet implemented] '
        'instead of the bucket dataset, if possible.',
        argstr='-cbucket %s')
    out_file = File(desc='output statistics file', argstr='-bucket %s')
    num_threads = traits.Int(
        desc='run the program with provided number of sub-processes',
        argstr='-jobs %d',
        nohash=True)
    fout = traits.Bool(
        desc='output F-statistic for each stimulus', argstr='-fout')
    rout = traits.Bool(
        desc='output the R^2 statistic for each stimulus', argstr='-rout')
    tout = traits.Bool(
        desc='output the T-statistic for each stimulus', argstr='-tout')
    vout = traits.Bool(
        desc='output the sample variance (MSE) for each stimulus',
        argstr='-vout')
    nofdr = traits.Bool(
        desc="Don't compute the statistic-vs-FDR curves for the bucket "
             "dataset.",
        argstr='-noFDR')
    global_times = traits.Bool(
        desc='use global timing for stimulus timing files',
        argstr='-global_times',
        xor=['local_times'])
    local_times = traits.Bool(
        desc='use local timing for stimulus timing files',
        argstr='-local_times',
        xor=['global_times'])
    num_stimts = traits.Int(
        desc='number of stimulus timing files',
        argstr='-num_stimts %d',
        position=-6)
    stim_times = traits.List(
        traits.Tuple(
            traits.Int(desc='k-th response model'),
            File(desc='stimulus timing file', exists=True),
            Str(desc='model')),
        desc='generate a response model from a set of stimulus times'
        ' given in file.',
        argstr='-stim_times %d %s \'%s\'...',
        position=-5)
    stim_label = traits.List(
        traits.Tuple(
            traits.Int(desc='k-th input stimulus'),
            Str(desc='stimulus label')),
        desc='label for kth input stimulus (e.g., Label1)',
        argstr='-stim_label %d %s...',
        requires=['stim_times'],
        position=-4)
    stim_times_subtract = traits.Float(
        desc='this option means to subtract specified seconds from each time '
        'encountered in any \'stim_times\' option. The purpose of this '
        'option is to make it simple to adjust timing files for the '
        'removal of images from the start of each imaging run.',
        argstr='-stim_times_subtract %f')
    num_glt = traits.Int(
        desc='number of general linear tests (i.e., contrasts)',
        argstr='-num_glt %d',
        position=-3)
    gltsym = traits.List(
        Str(desc='symbolic general linear test'),
        desc='general linear tests (i.e., contrasts) using symbolic '
        'conventions (e.g., \'+Label1 -Label2\')',
        argstr='-gltsym \'SYM: %s\'...',
        position=-2)
    glt_label = traits.List(
        traits.Tuple(
            traits.Int(desc='k-th general linear test'),
            Str(desc='GLT label')),
        desc='general linear test (i.e., contrast) labels',
        argstr='-glt_label %d %s...',
        requires=['gltsym'],
        position=-1)


class DeconvolveOutputSpec(TraitedSpec):
    out_file = File(desc='output statistics file', exists=True)
    reml_script = File(
        desc='automatical generated script to run 3dREMLfit', exists=True)
    x1D = File(desc='save out X matrix', exists=True)
    cbucket = File(desc='output regression coefficients file (if generated)')


class Deconvolve(AFNICommand):
    """Performs OLS regression given a 4D neuroimage file and stimulus timings

    For complete details, see the `3dDeconvolve Documentation.
    <https://afni.nimh.nih.gov/pub/dist/doc/program_help/3dDeconvolve.html>`_

    Examples
    ========

    >>> from nipype.interfaces import afni
    >>> deconvolve = afni.Deconvolve()
    >>> deconvolve.inputs.in_files = ['functional.nii', 'functional2.nii']
    >>> deconvolve.inputs.out_file = 'output.nii'
    >>> deconvolve.inputs.x1D = 'output.1D'
    >>> stim_times = [(1, 'timeseries.txt', 'SPMG1(4)')]
    >>> deconvolve.inputs.stim_times = stim_times
    >>> deconvolve.inputs.stim_label = [(1, 'Houses')]
    >>> deconvolve.inputs.gltsym = ['SYM: +Houses']
    >>> deconvolve.inputs.glt_label = [(1, 'Houses')]
    >>> deconvolve.cmdline
    "3dDeconvolve -input functional.nii functional2.nii -bucket output.nii -x1D output.1D -num_stimts 1 -stim_times 1 timeseries.txt 'SPMG1(4)' -stim_label 1 Houses -num_glt 1 -gltsym 'SYM: +Houses' -glt_label 1 Houses"
    >>> res = deconvolve.run()  # doctest: +SKIP
    """

    _cmd = '3dDeconvolve'
    input_spec = DeconvolveInputSpec
    output_spec = DeconvolveOutputSpec

    def _format_arg(self, name, trait_spec, value):
        if name == 'gltsym':
            for n, val in enumerate(value):
                if val.startswith('SYM: '):
                    value[n] = val.lstrip('SYM: ')

        return super(Deconvolve, self)._format_arg(name, trait_spec, value)

    def _parse_inputs(self, skip=None):
        if skip is None:
            skip = []
        if len(self.inputs.stim_times) and not isdefined(
                self.inputs.num_stimts):
            self.inputs.num_stimts = len(self.inputs.stim_times)
        if len(self.inputs.gltsym) and not isdefined(self.inputs.num_glt):
            self.inputs.num_glt = len(self.inputs.gltsym)
        if not isdefined(self.inputs.out_file):
            self.inputs.out_file = 'Decon.nii'

        return super(Deconvolve, self)._parse_inputs(skip)

    def _list_outputs(self):
        outputs = self.output_spec().get()

        _gen_fname_opts = {}
        _gen_fname_opts['basename'] = self.inputs.out_file
        _gen_fname_opts['cwd'] = os.getcwd()

        if isdefined(self.inputs.x1D):
            if not self.inputs.x1D.endswith('.xmat.1D'):
                outputs['x1D'] = os.path.abspath(self.inputs.x1D + '.xmat.1D')
            else:
                outputs['x1D'] = os.path.abspath(self.inputs.x1D)
        else:
            outputs['x1D'] = self._gen_fname(
                suffix='.xmat.1D', **_gen_fname_opts)

        if isdefined(self.inputs.cbucket):
            outputs['cbucket'] = os.path.abspath(self.inputs.cbucket)

        outputs['reml_script'] = self._gen_fname(
            suffix='.REML_cmd', **_gen_fname_opts)
        # remove out_file from outputs if x1d_stop set to True
        if self.inputs.x1D_stop:
            del outputs['out_file'], outputs['cbucket']
        else:
            outputs['out_file'] = os.path.abspath(self.inputs.out_file)

        return outputs


class RemlfitInputSpec(AFNICommandInputSpec):
    # mandatory files
    in_files = InputMultiPath(
        File(exists=True),
        desc='Read time series dataset',
        argstr='-input "%s"',
        mandatory=True,
        copyfile=False,
        sep=" ")
    matrix = File(
        desc='the design matrix file, which should have been output from '
        'Deconvolve via the \'x1D\' option',
        argstr='-matrix %s',
        mandatory=True)
    # "Semi-Hidden Alternative Ways to Define the Matrix"
    polort = traits.Int(
        desc='if no \'matrix\' option is given, AND no \'matim\' option, '
        'create a matrix with Legendre polynomial regressors'
        'up to the specified order. The default value is 0, which'
        'produces a matrix with a single column of all ones',
        argstr='-polort %d',
        xor=['matrix'])
    matim = File(
        desc='read a standard file as the matrix. You can use only Col as '
        'a name in GLTs with these nonstandard matrix input methods, '
        'since the other names come from the \'matrix\' file. '
        'These mutually exclusive options are ignored if \'matrix\' '
        'is used.',
        argstr='-matim %s',
        xor=['matrix'])
    # Other arguments
    mask = File(
        desc='filename of 3D mask dataset; only data time series from within '
        'the mask will be analyzed; results for voxels outside the mask '
        'will be set to zero.',
        argstr='-mask %s',
        exists=True)
    automask = traits.Bool(
        usedefault=True,
        argstr='-automask',
        desc='build a mask automatically from input data (will be slow for '
        'long time series datasets)')
    STATmask = File(
        desc='filename of 3D mask dataset to be used for the purpose '
        'of reporting truncation-to float issues AND for computing the '
        'FDR curves. The actual results ARE not masked with this option '
        '(only with \'mask\' or \'automask\' options).',
        argstr='-STATmask %s',
        exists=True)
    addbase = InputMultiPath(
        File(
            exists=True,
            desc='file containing columns to add to regression matrix'),
        desc='file(s) to add baseline model columns to the matrix with this '
        'option. Each column in the specified file(s) will be appended '
        'to the matrix. File(s) must have at least as many rows as the '
        'matrix does.',
        copyfile=False,
        sep=" ",
        argstr='-addbase %s')
    slibase = InputMultiPath(
        File(
            exists=True,
            desc='file containing columns to add to regression matrix'),
        desc='similar to \'addbase\' in concept, BUT each specified file '
        'must have an integer multiple of the number of slices '
        'in the input dataset(s); then, separate regression '
        'matrices are generated for each slice, with the '
        'first column of the file appended to the matrix for '
        'the first slice of the dataset, the second column of the file '
        'appended to the matrix for the first slice of the dataset, '
        'and so on. Intended to help model physiological noise in FMRI, '
        'or other effects you want to regress out that might '
        'change significantly in the inter-slice time intervals. This '
        'will slow the program down, and make it use a lot more memory '
        '(to hold all the matrix stuff).',
        argstr='-slibase %s')
    slibase_sm = InputMultiPath(
        File(
            exists=True,
            desc='file containing columns to add to regression matrix'),
        desc='similar to \'slibase\', BUT each file much be in slice major '
        'order (i.e. all slice0 columns come first, then all slice1 '
        'columns, etc).',
        argstr='-slibase_sm %s')
    usetemp = traits.Bool(
        desc='write intermediate stuff to disk, to economize on RAM. '
        'Using this option might be necessary to run with '
        '\'slibase\' and with \'Grid\' values above the default, '
        'since the program has to store a large number of '
        'matrices for such a problem: two for every slice and '
        'for every (a,b) pair in the ARMA parameter grid. Temporary '
        'files are written to the directory given in environment '
        'variable TMPDIR, or in /tmp, or in ./ (preference is in that '
        'order)',
        argstr='-usetemp')
    nodmbase = traits.Bool(
        desc='by default, baseline columns added to the matrix via '
        '\'addbase\' or \'slibase\' or \'dsort\' will each have their '
        'mean removed (as is done in Deconvolve); this option turns this '
        'centering off',
        argstr='-nodmbase',
        requires=['addbase', 'dsort'])
    dsort = File(
        desc='4D dataset to be used as voxelwise baseline regressor',
        exists=True,
        copyfile=False,
        argstr='-dsort %s')
    dsort_nods = traits.Bool(
        desc='if \'dsort\' option is used, this command will output '
        'additional results files excluding the \'dsort\' file',
        argstr='-dsort_nods',
        requires=['dsort'])
    fout = traits.Bool(
        desc='output F-statistic for each stimulus', argstr='-fout')
    rout = traits.Bool(
        desc='output the R^2 statistic for each stimulus', argstr='-rout')
    tout = traits.Bool(
        desc='output the T-statistic for each stimulus; if you use '
        '\'out_file\' and do not give any of \'fout\', \'tout\','
        'or \'rout\', then the program assumes \'fout\' is activated.',
        argstr='-tout')
    nofdr = traits.Bool(
        desc='do NOT add FDR curve data to bucket datasets; FDR curves can '
        'take a long time if \'tout\' is used',
        argstr='-noFDR')
    nobout = traits.Bool(
        desc='do NOT add baseline (null hypothesis) regressor betas '
        'to the \'rbeta_file\' and/or \'obeta_file\' output datasets.',
        argstr='-nobout')
    gltsym = traits.List(
        traits.Either(
            traits.Tuple(File(exists=True), Str()), traits.Tuple(Str(),
                                                                 Str())),
        desc='read a symbolic GLT from input file and associate it with a '
        'label. As in Deconvolve, you can also use the \'SYM:\' method '
        'to provide the definition of the GLT directly as a string '
        '(e.g., with \'SYM: +Label1 -Label2\'). Unlike Deconvolve, you '
        'MUST specify \'SYM: \' if providing the GLT directly as a '
        'string instead of from a file',
        argstr='-gltsym "%s" %s...')
    out_file = File(
        desc='output dataset for beta + statistics from the REML estimation; '
        'also contains the results of any GLT analysis requested '
        'in the Deconvolve setup, similar to the \'bucket\' output '
        'from Deconvolve. This dataset does NOT get the betas '
        '(or statistics) of those regressors marked as \'baseline\' '
        'in the matrix file.',
        argstr='-Rbuck %s')
    var_file = File(
        desc='output dataset for REML variance parameters', argstr='-Rvar %s')
    rbeta_file = File(
        desc='output dataset for beta weights from the REML estimation, '
        'similar to the \'cbucket\' output from Deconvolve. This dataset '
        'will contain all the beta weights, for baseline and stimulus '
        'regressors alike, unless the \'-nobout\' option is given -- '
        'in that case, this dataset will only get the betas for the '
        'stimulus regressors.',
        argstr='-Rbeta %s')
    glt_file = File(
        desc='output dataset for beta + statistics from the REML estimation, '
        'but ONLY for the GLTs added on the REMLfit command line itself '
        'via \'gltsym\'; GLTs from Deconvolve\'s command line will NOT '
        'be included.',
        argstr='-Rglt %s')
    fitts_file = File(
        desc='ouput dataset for REML fitted model', argstr='-Rfitts %s')
    errts_file = File(
        desc='output dataset for REML residuals = data - fitted model',
        argstr='-Rerrts %s')
    wherr_file = File(
        desc='dataset for REML residual, whitened using the estimated '
        'ARMA(1,1) correlation matrix of the noise',
        argstr='-Rwherr %s')
    quiet = traits.Bool(
        desc='turn off most progress messages', argstr='-quiet')
    verb = traits.Bool(
        desc='turns on more progress messages, including memory usage '
        'progress reports at various stages',
        argstr='-verb')
    goforit = traits.Bool(
        desc='With potential issues flagged in the design matrix, an attempt '
        'will nevertheless be made to fit the model',
        argstr='-GOFORIT')
    ovar = File(
        desc='dataset for OLSQ st.dev. parameter (kind of boring)',
        argstr='-Ovar %s')
    obeta = File(
        desc='dataset for beta weights from the OLSQ estimation',
        argstr='-Obeta %s')
    obuck = File(
        desc='dataset for beta + statistics from the OLSQ estimation',
        argstr='-Obuck %s')
    oglt = File(
        desc='dataset for beta + statistics from \'gltsym\' options',
        argstr='-Oglt %s')
    ofitts = File(desc='dataset for OLSQ fitted model', argstr='-Ofitts %s')
    oerrts = File(
        desc='dataset for OLSQ residuals (data - fitted model)',
        argstr='-Oerrts %s')


class RemlfitOutputSpec(AFNICommandOutputSpec):
    out_file = File(
        desc='dataset for beta + statistics from the REML estimation (if '
        'generated')
    var_file = File(desc='dataset for REML variance parameters (if generated)')
    rbeta_file = File(
        desc='dataset for beta weights from the REML estimation (if '
        'generated)')
    rbeta_file = File(
        desc='output dataset for beta weights from the REML estimation (if '
        'generated')
    glt_file = File(
        desc='output dataset for beta + statistics from the REML estimation, '
        'but ONLY for the GLTs added on the REMLfit command '
        'line itself via \'gltsym\' (if generated)')
    fitts_file = File(
        desc='ouput dataset for REML fitted model (if generated)')
    errts_file = File(
        desc='output dataset for REML residuals = data - fitted model (if '
        'generated')
    wherr_file = File(
        desc='dataset for REML residual, whitened using the estimated '
        'ARMA(1,1) correlation matrix of the noise (if generated)')
    ovar = File(desc='dataset for OLSQ st.dev. parameter (if generated)')
    obeta = File(desc='dataset for beta weights from the OLSQ estimation (if '
                 'generated)')
    obuck = File(
        desc='dataset for beta + statistics from the OLSQ estimation (if '
        'generated)')
    oglt = File(
        desc='dataset for beta + statistics from \'gltsym\' options (if '
        'generated')
    ofitts = File(desc='dataset for OLSQ fitted model (if generated)')
    oerrts = File(desc='dataset for OLSQ residuals = data - fitted model (if '
                  'generated')


class Remlfit(AFNICommand):
    """Performs Generalized least squares time series fit with Restricted
    Maximum Likelihood (REML) estimation of the temporal auto-correlation
    structure.

    For complete details, see the `3dREMLfit Documentation.
    <https://afni.nimh.nih.gov/pub/dist/doc/program_help/3dREMLfit.html>`_

    Examples
    ========

    >>> from nipype.interfaces import afni
    >>> remlfit = afni.Remlfit()
    >>> remlfit.inputs.in_files = ['functional.nii', 'functional2.nii']
    >>> remlfit.inputs.out_file = 'output.nii'
    >>> remlfit.inputs.matrix = 'output.1D'
    >>> remlfit.inputs.gltsym = [('SYM: +Lab1 -Lab2', 'TestSYM'), ('timeseries.txt', 'TestFile')]
    >>> remlfit.cmdline
    '3dREMLfit -gltsym "SYM: +Lab1 -Lab2" TestSYM -gltsym "timeseries.txt" TestFile -input "functional.nii functional2.nii" -matrix output.1D -Rbuck output.nii'
    >>> res = remlfit.run()  # doctest: +SKIP
    """

    _cmd = '3dREMLfit'
    input_spec = RemlfitInputSpec
    output_spec = RemlfitOutputSpec

    def _parse_inputs(self, skip=None):
        if skip is None:
            skip = []
        return super(Remlfit, self)._parse_inputs(skip)

    def _list_outputs(self):
        outputs = self.output_spec().get()

        for key in outputs.keys():
            if isdefined(self.inputs.get()[key]):
                outputs[key] = os.path.abspath(self.inputs.get()[key])

        return outputs


class SynthesizeInputSpec(AFNICommandInputSpec):
    cbucket = File(
        desc='Read the dataset output from '
        '3dDeconvolve via the \'-cbucket\' option.',
        argstr='-cbucket %s',
        copyfile=False,
        mandatory=True)
    matrix = File(
        desc='Read the matrix output from '
        '3dDeconvolve via the \'-x1D\' option.',
        argstr='-matrix %s',
        copyfile=False,
        mandatory=True)
    select = traits.List(
        Str(desc='selected columns to synthesize'),
        argstr='-select %s',
        desc='A list of selected columns from the matrix (and the '
        'corresponding coefficient sub-bricks from the '
        'cbucket). Valid types include \'baseline\', '
        ' \'polort\', \'allfunc\', \'allstim\', \'all\', '
        'Can also provide \'something\' where something matches '
        'a stim_label from 3dDeconvolve, and \'digits\' where digits '
        'are the numbers of the select matrix columns by '
        'numbers (starting at 0), or number ranges of the form '
        '\'3..7\' and \'3-7\'.',
        mandatory=True)
    out_file = File(
        name_template='syn',
        desc='output dataset prefix name (default \'syn\')',
        argstr='-prefix %s')
    dry_run = traits.Bool(
        desc='Don\'t compute the output, just '
        'check the inputs.',
        argstr='-dry')
    TR = traits.Float(
        desc='TR to set in the output.  The default value of '
        'TR is read from the header of the matrix file.',
        argstr='-TR %f')
    cenfill = traits.Enum(
        'zero',
        'nbhr',
        'none',
        argstr='-cenfill %s',
        desc='Determines how censored time points from the '
        '3dDeconvolve run will be filled. Valid types '
        'are \'zero\', \'nbhr\' and \'none\'.')


class Synthesize(AFNICommand):
    """Reads a '-cbucket' dataset and a '.xmat.1D' matrix from 3dDeconvolve,
       and synthesizes a fit dataset using user-selected sub-bricks and
       matrix columns.

    For complete details, see the `3dSynthesize Documentation.
    <https://afni.nimh.nih.gov/pub/dist/doc/program_help/3dSynthesize.html>`_

    Examples
    ========

    >>> from nipype.interfaces import afni
    >>> synthesize = afni.Synthesize()
    >>> synthesize.inputs.cbucket = 'functional.nii'
    >>> synthesize.inputs.matrix = 'output.1D'
    >>> synthesize.inputs.select = ['baseline']
    >>> synthesize.cmdline
    '3dSynthesize -cbucket functional.nii -matrix output.1D -select baseline'
    >>> syn = synthesize.run()  # doctest: +SKIP
    """

    _cmd = '3dSynthesize'
    input_spec = SynthesizeInputSpec
    output_spec = AFNICommandOutputSpec

    def _list_outputs(self):
        outputs = self.output_spec().get()

        for key in outputs.keys():
            if isdefined(self.inputs.get()[key]):
                outputs[key] = os.path.abspath(self.inputs.get()[key])

        return outputs



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..developer import JistLaminarVolumetricLayering


def test_JistLaminarVolumetricLayering_inputs():
    input_map = dict(
        args=dict(argstr='%s', ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        inInner=dict(
            argstr='--inInner %s',
            extensions=None,
        ),
        inLayering=dict(argstr='--inLayering %s', ),
        inLayering2=dict(argstr='--inLayering2 %s', ),
        inMax=dict(argstr='--inMax %d', ),
        inMin=dict(argstr='--inMin %f', ),
        inNumber=dict(argstr='--inNumber %d', ),
        inOuter=dict(
            argstr='--inOuter %s',
            extensions=None,
        ),
        inTopology=dict(argstr='--inTopology %s', ),
        incurvature=dict(argstr='--incurvature %d', ),
        inpresmooth=dict(argstr='--inpresmooth %s', ),
        inratio=dict(argstr='--inratio %f', ),
        null=dict(argstr='--null %s', ),
        outContinuous=dict(
            argstr='--outContinuous %s',
            hash_files=False,
        ),
        outDiscrete=dict(
            argstr='--outDiscrete %s',
            hash_files=False,
        ),
        outLayer=dict(
            argstr='--outLayer %s',
            hash_files=False,
        ),
        xDefaultMem=dict(argstr='-xDefaultMem %d', ),
        xMaxProcess=dict(
            argstr='-xMaxProcess %d',
            usedefault=True,
        ),
        xPrefExt=dict(argstr='--xPrefExt %s', ),
    )
    inputs = JistLaminarVolumetricLayering.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_JistLaminarVolumetricLayering_outputs():
    output_map = dict(
        outContinuous=dict(extensions=None, ),
        outDiscrete=dict(extensions=None, ),
        outLayer=dict(extensions=None, ),
    )
    outputs = JistLaminarVolumetricLayering.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..epi import Eddy


def test_Eddy_inputs():
    input_map = dict(
        args=dict(argstr='%s', ),
        cnr_maps=dict(
            argstr='--cnr_maps',
            min_ver='5.0.10',
        ),
        dont_peas=dict(argstr='--dont_peas', ),
        dont_sep_offs_move=dict(argstr='--dont_sep_offs_move', ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        fep=dict(argstr='--fep', ),
        field=dict(argstr='--field=%s', ),
        field_mat=dict(
            argstr='--field_mat=%s',
            extensions=None,
        ),
        flm=dict(argstr='--flm=%s', ),
        fudge_factor=dict(
            argstr='--ff=%s',
            usedefault=True,
        ),
        fwhm=dict(argstr='--fwhm=%s', ),
        in_acqp=dict(
            argstr='--acqp=%s',
            extensions=None,
            mandatory=True,
        ),
        in_bval=dict(
            argstr='--bvals=%s',
            extensions=None,
            mandatory=True,
        ),
        in_bvec=dict(
            argstr='--bvecs=%s',
            extensions=None,
            mandatory=True,
        ),
        in_file=dict(
            argstr='--imain=%s',
            extensions=None,
            mandatory=True,
        ),
        in_index=dict(
            argstr='--index=%s',
            extensions=None,
            mandatory=True,
        ),
        in_mask=dict(
            argstr='--mask=%s',
            extensions=None,
            mandatory=True,
        ),
        in_topup_fieldcoef=dict(
            argstr='--topup=%s',
            extensions=None,
            requires=['in_topup_movpar'],
        ),
        in_topup_movpar=dict(
            extensions=None,
            requires=['in_topup_fieldcoef'],
        ),
        interp=dict(argstr='--interp=%s', ),
        is_shelled=dict(argstr='--data_is_shelled', ),
        method=dict(argstr='--resamp=%s', ),
        niter=dict(
            argstr='--niter=%s',
            usedefault=True,
        ),
        num_threads=dict(
            nohash=True,
            usedefault=True,
        ),
        nvoxhp=dict(
            argstr='--nvoxhp=%s',
            usedefault=True,
        ),
        out_base=dict(
            argstr='--out=%s',
            usedefault=True,
        ),
        output_type=dict(),
        repol=dict(argstr='--repol', ),
        residuals=dict(
            argstr='--residuals',
            min_ver='5.0.10',
        ),
        session=dict(
            argstr='--session=%s',
            extensions=None,
        ),
        slm=dict(argstr='--slm=%s', ),
        use_cuda=dict(),
    )
    inputs = Eddy.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_Eddy_outputs():
    output_map = dict(
        out_cnr_maps=dict(extensions=None, ),
        out_corrected=dict(extensions=None, ),
        out_movement_rms=dict(extensions=None, ),
        out_outlier_report=dict(extensions=None, ),
        out_parameter=dict(extensions=None, ),
        out_residuals=dict(extensions=None, ),
        out_restricted_movement_rms=dict(extensions=None, ),
        out_rotated_bvecs=dict(extensions=None, ),
        out_shell_alignment_parameters=dict(extensions=None, ),
    )
    outputs = Eddy.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..brains import BRAINSAlignMSP


def test_BRAINSAlignMSP_inputs():
    input_map = dict(
        BackgroundFillValue=dict(argstr='--BackgroundFillValue %s', ),
        OutputresampleMSP=dict(
            argstr='--OutputresampleMSP %s',
            hash_files=False,
        ),
        args=dict(argstr='%s', ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        inputVolume=dict(
            argstr='--inputVolume %s',
            extensions=None,
        ),
        interpolationMode=dict(argstr='--interpolationMode %s', ),
        mspQualityLevel=dict(argstr='--mspQualityLevel %d', ),
        numberOfThreads=dict(argstr='--numberOfThreads %d', ),
        rescaleIntensities=dict(argstr='--rescaleIntensities ', ),
        rescaleIntensitiesOutputRange=dict(
            argstr='--rescaleIntensitiesOutputRange %s',
            sep=',',
        ),
        resultsDir=dict(
            argstr='--resultsDir %s',
            hash_files=False,
        ),
        trimRescaledIntensities=dict(argstr='--trimRescaledIntensities %f', ),
        verbose=dict(argstr='--verbose ', ),
        writedebuggingImagesLevel=dict(
            argstr='--writedebuggingImagesLevel %d', ),
    )
    inputs = BRAINSAlignMSP.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_BRAINSAlignMSP_outputs():
    output_map = dict(
        OutputresampleMSP=dict(extensions=None, ),
        resultsDir=dict(),
    )
    outputs = BRAINSAlignMSP.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..regutils import RegTools


def test_RegTools_inputs():
    input_map = dict(
        add_val=dict(argstr='-add %s', ),
        args=dict(argstr='%s', ),
        bin_flag=dict(argstr='-bin', ),
        chg_res_val=dict(argstr='-chgres %f %f %f', ),
        div_val=dict(argstr='-div %s', ),
        down_flag=dict(argstr='-down', ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        in_file=dict(
            argstr='-in %s',
            extensions=None,
            mandatory=True,
        ),
        inter_val=dict(argstr='-interp %d', ),
        iso_flag=dict(argstr='-iso', ),
        mask_file=dict(
            argstr='-nan %s',
            extensions=None,
        ),
        mul_val=dict(argstr='-mul %s', ),
        noscl_flag=dict(argstr='-noscl', ),
        omp_core_val=dict(
            argstr='-omp %i',
            usedefault=True,
        ),
        out_file=dict(
            argstr='-out %s',
            extensions=None,
            name_source=['in_file'],
            name_template='%s_tools.nii.gz',
        ),
        rms_val=dict(
            argstr='-rms %s',
            extensions=None,
        ),
        smo_g_val=dict(argstr='-smoG %f %f %f', ),
        smo_s_val=dict(argstr='-smoS %f %f %f', ),
        sub_val=dict(argstr='-sub %s', ),
        thr_val=dict(argstr='-thr %f', ),
    )
    inputs = RegTools.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_RegTools_outputs():
    output_map = dict(out_file=dict(extensions=None, ), )
    outputs = RegTools.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# -*- coding: utf-8 -*-
from __future__ import (print_function, division, unicode_literals,
                        absolute_import)

import nibabel as nb

from ... import logging
from ..base import TraitedSpec, File, isdefined
from .base import DipyDiffusionInterface, DipyBaseInterfaceInputSpec

IFLOGGER = logging.getLogger('nipype.interface')


class DTIInputSpec(DipyBaseInterfaceInputSpec):
    mask_file = File(exists=True, desc='An optional white matter mask')


class DTIOutputSpec(TraitedSpec):
    out_file = File(exists=True)
    fa_file = File(exists=True)
    md_file = File(exists=True)
    rd_file = File(exists=True)
    ad_file = File(exists=True)
    color_fa_file = File(exists=True)


class DTI(DipyDiffusionInterface):
    """
    Calculates the diffusion tensor model parameters

    Example
    -------

    >>> import nipype.interfaces.dipy as dipy
    >>> dti = dipy.DTI()
    >>> dti.inputs.in_file = 'diffusion.nii'
    >>> dti.inputs.in_bvec = 'bvecs'
    >>> dti.inputs.in_bval = 'bvals'
    >>> dti.run()                                   # doctest: +SKIP
    """
    input_spec = DTIInputSpec
    output_spec = DTIOutputSpec

    def _run_interface(self, runtime):
        from dipy.reconst import dti
        from dipy.io.utils import nifti1_symmat
        gtab = self._get_gradient_table()

        img = nb.load(self.inputs.in_file)
        data = img.get_data()
        affine = img.affine
        mask = None
        if isdefined(self.inputs.mask_file):
            mask = nb.load(self.inputs.mask_file).get_data()

        # Fit it
        tenmodel = dti.TensorModel(gtab)
        ten_fit = tenmodel.fit(data, mask)
        lower_triangular = ten_fit.lower_triangular()
        img = nifti1_symmat(lower_triangular, affine)
        out_file = self._gen_filename('dti')
        nb.save(img, out_file)
        IFLOGGER.info('DTI parameters image saved as %s', out_file)

        # FA MD RD and AD
        for metric in ["fa", "md", "rd", "ad", "color_fa"]:
            data = getattr(ten_fit, metric).astype("float32")
            out_name = self._gen_filename(metric)
            nb.Nifti1Image(data, affine).to_filename(out_name)
            IFLOGGER.info('DTI %s image saved as %s', metric, out_name)

        return runtime

    def _list_outputs(self):
        outputs = self._outputs().get()
        outputs['out_file'] = self._gen_filename('dti')

        for metric in ["fa", "md", "rd", "ad", "color_fa"]:
            outputs["{}_file".format(metric)] = self._gen_filename(metric)

        return outputs


class TensorModeInputSpec(DipyBaseInterfaceInputSpec):
    mask_file = File(exists=True, desc='An optional white matter mask')


class TensorModeOutputSpec(TraitedSpec):
    out_file = File(exists=True)


class TensorMode(DipyDiffusionInterface):
    """
    Creates a map of the mode of the diffusion tensors given a set of
    diffusion-weighted images, as well as their associated b-values and
    b-vectors. Fits the diffusion tensors and calculates tensor mode
    with Dipy.

    .. [1] Daniel B. Ennis and G. Kindlmann, "Orthogonal Tensor
        Invariants and the Analysis of Diffusion Tensor Magnetic Resonance
        Images", Magnetic Resonance in Medicine, vol. 55, no. 1, pp. 136-146,
        2006.

    Example
    -------

    >>> import nipype.interfaces.dipy as dipy
    >>> mode = dipy.TensorMode()
    >>> mode.inputs.in_file = 'diffusion.nii'
    >>> mode.inputs.in_bvec = 'bvecs'
    >>> mode.inputs.in_bval = 'bvals'
    >>> mode.run()                                   # doctest: +SKIP
    """
    input_spec = TensorModeInputSpec
    output_spec = TensorModeOutputSpec

    def _run_interface(self, runtime):
        from dipy.reconst import dti

        # Load the 4D image files
        img = nb.load(self.inputs.in_file)
        data = img.get_data()
        affine = img.affine

        # Load the gradient strengths and directions
        gtab = self._get_gradient_table()

        # Mask the data so that tensors are not fit for
        # unnecessary voxels
        mask = data[..., 0] > 50

        # Fit the tensors to the data
        tenmodel = dti.TensorModel(gtab)
        tenfit = tenmodel.fit(data, mask)

        # Calculate the mode of each voxel's tensor
        mode_data = tenfit.mode

        # Write as a 3D Nifti image with the original affine
        img = nb.Nifti1Image(mode_data, affine)
        out_file = self._gen_filename('mode')
        nb.save(img, out_file)
        IFLOGGER.info('Tensor mode image saved as %s', out_file)
        return runtime

    def _list_outputs(self):
        outputs = self._outputs().get()
        outputs['out_file'] = self._gen_filename('mode')
        return outputs



# -*- coding: utf-8 -*-
# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
"""
Package contains interfaces for using existing functionality in other packages

Requires Packages to be installed
"""

from .base import (IdentityInterface, Rename, Select, Split, Merge,
                   AssertEqual)
from .csv import CSVReader
from .wrappers import Function



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..gtract import gtractFiberTracking


def test_gtractFiberTracking_inputs():
    input_map = dict(
        args=dict(argstr='%s', ),
        branchingAngle=dict(argstr='--branchingAngle %f', ),
        branchingThreshold=dict(argstr='--branchingThreshold %f', ),
        curvatureThreshold=dict(argstr='--curvatureThreshold %f', ),
        endingSeedsLabel=dict(argstr='--endingSeedsLabel %d', ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        guidedCurvatureThreshold=dict(
            argstr='--guidedCurvatureThreshold %f', ),
        inputAnisotropyVolume=dict(
            argstr='--inputAnisotropyVolume %s',
            extensions=None,
        ),
        inputEndingSeedsLabelMapVolume=dict(
            argstr='--inputEndingSeedsLabelMapVolume %s',
            extensions=None,
        ),
        inputStartingSeedsLabelMapVolume=dict(
            argstr='--inputStartingSeedsLabelMapVolume %s',
            extensions=None,
        ),
        inputTensorVolume=dict(
            argstr='--inputTensorVolume %s',
            extensions=None,
        ),
        inputTract=dict(
            argstr='--inputTract %s',
            extensions=None,
        ),
        maximumBranchPoints=dict(argstr='--maximumBranchPoints %d', ),
        maximumGuideDistance=dict(argstr='--maximumGuideDistance %f', ),
        maximumLength=dict(argstr='--maximumLength %f', ),
        minimumLength=dict(argstr='--minimumLength %f', ),
        numberOfThreads=dict(argstr='--numberOfThreads %d', ),
        outputTract=dict(
            argstr='--outputTract %s',
            hash_files=False,
        ),
        randomSeed=dict(argstr='--randomSeed %d', ),
        seedThreshold=dict(argstr='--seedThreshold %f', ),
        startingSeedsLabel=dict(argstr='--startingSeedsLabel %d', ),
        stepSize=dict(argstr='--stepSize %f', ),
        tendF=dict(argstr='--tendF %f', ),
        tendG=dict(argstr='--tendG %f', ),
        trackingMethod=dict(argstr='--trackingMethod %s', ),
        trackingThreshold=dict(argstr='--trackingThreshold %f', ),
        useLoopDetection=dict(argstr='--useLoopDetection ', ),
        useRandomWalk=dict(argstr='--useRandomWalk ', ),
        useTend=dict(argstr='--useTend ', ),
        writeXMLPolyDataFile=dict(argstr='--writeXMLPolyDataFile ', ),
    )
    inputs = gtractFiberTracking.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_gtractFiberTracking_outputs():
    output_map = dict(outputTract=dict(extensions=None, ), )
    outputs = gtractFiberTracking.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..lesions import FillLesions


def test_FillLesions_inputs():
    input_map = dict(
        args=dict(argstr='%s', ),
        bin_mask=dict(
            argstr='-mask %s',
            extensions=None,
        ),
        cwf=dict(argstr='-cwf %f', ),
        debug=dict(argstr='-debug', ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        in_dilation=dict(argstr='-dil %d', ),
        in_file=dict(
            argstr='-i %s',
            extensions=None,
            mandatory=True,
            position=1,
        ),
        lesion_mask=dict(
            argstr='-l %s',
            extensions=None,
            mandatory=True,
            position=2,
        ),
        match=dict(argstr='-match %f', ),
        other=dict(argstr='-other', ),
        out_datatype=dict(argstr='-odt %s', ),
        out_file=dict(
            argstr='-o %s',
            extensions=None,
            name_source=['in_file'],
            name_template='%s_lesions_filled.nii.gz',
            position=3,
        ),
        search=dict(argstr='-search %f', ),
        size=dict(argstr='-size %d', ),
        smooth=dict(argstr='-smo %f', ),
        use_2d=dict(argstr='-2D', ),
        verbose=dict(argstr='-v', ),
    )
    inputs = FillLesions.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_FillLesions_outputs():
    output_map = dict(out_file=dict(extensions=None, ), )
    outputs = FillLesions.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# -*- coding: utf-8 -*-
# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
import os

import pytest
from nipype.testing.fixtures import create_files_in_directory

import nipype.interfaces.spm as spm
from nipype.interfaces.spm import no_spm
import nipype.interfaces.matlab as mlab

mlab.MatlabCommand.set_default_matlab_cmd(os.getenv('MATLABCMD', 'matlab'))


def test_slicetiming():
    assert spm.SliceTiming._jobtype == 'temporal'
    assert spm.SliceTiming._jobname == 'st'


def test_slicetiming_list_outputs(create_files_in_directory):
    filelist, outdir = create_files_in_directory
    st = spm.SliceTiming(in_files=filelist[0])
    assert st._list_outputs()['timecorrected_files'][0][0] == 'a'


def test_realign():
    assert spm.Realign._jobtype == 'spatial'
    assert spm.Realign._jobname == 'realign'
    assert spm.Realign().inputs.jobtype == 'estwrite'


def test_realign_list_outputs(create_files_in_directory):
    filelist, outdir = create_files_in_directory
    rlgn = spm.Realign(in_files=filelist[0])
    assert rlgn._list_outputs()['realignment_parameters'][0].startswith('rp_')
    assert rlgn._list_outputs()['realigned_files'][0].startswith('r')
    assert rlgn._list_outputs()['mean_image'].startswith('mean')


def test_coregister():
    assert spm.Coregister._jobtype == 'spatial'
    assert spm.Coregister._jobname == 'coreg'
    assert spm.Coregister().inputs.jobtype == 'estwrite'


def test_coregister_list_outputs(create_files_in_directory):
    filelist, outdir = create_files_in_directory
    coreg = spm.Coregister(source=filelist[0])
    assert coreg._list_outputs()['coregistered_source'][0].startswith('r')
    coreg = spm.Coregister(source=filelist[0], apply_to_files=filelist[1])
    assert coreg._list_outputs()['coregistered_files'][0].startswith('r')


def test_normalize():
    assert spm.Normalize._jobtype == 'spatial'
    assert spm.Normalize._jobname == 'normalise'
    assert spm.Normalize().inputs.jobtype == 'estwrite'


def test_normalize_list_outputs(create_files_in_directory):
    filelist, outdir = create_files_in_directory
    norm = spm.Normalize(source=filelist[0])
    assert norm._list_outputs()['normalized_source'][0].startswith('w')
    norm = spm.Normalize(source=filelist[0], apply_to_files=filelist[1])
    assert norm._list_outputs()['normalized_files'][0].startswith('w')


def test_normalize12():
    assert spm.Normalize12._jobtype == 'spatial'
    assert spm.Normalize12._jobname == 'normalise'
    assert spm.Normalize12().inputs.jobtype == 'estwrite'


def test_normalize12_list_outputs(create_files_in_directory):
    filelist, outdir = create_files_in_directory
    norm12 = spm.Normalize12(image_to_align=filelist[0])
    assert norm12._list_outputs()['normalized_image'][0].startswith('w')
    norm12 = spm.Normalize12(
        image_to_align=filelist[0], apply_to_files=filelist[1])
    assert norm12._list_outputs()['normalized_files'][0].startswith('w')


@pytest.mark.skipif(no_spm(), reason="spm is not installed")
def test_segment():
    if spm.Info.name() == "SPM12":
        assert spm.Segment()._jobtype == 'tools'
        assert spm.Segment()._jobname == 'oldseg'
    else:
        assert spm.Segment()._jobtype == 'spatial'
        assert spm.Segment()._jobname == 'preproc'


@pytest.mark.skipif(no_spm(), reason="spm is not installed")
def test_newsegment():
    if spm.Info.name() == "SPM12":
        assert spm.NewSegment()._jobtype == 'spatial'
        assert spm.NewSegment()._jobname == 'preproc'
    else:
        assert spm.NewSegment()._jobtype == 'tools'
        assert spm.NewSegment()._jobname == 'preproc8'


def test_smooth():
    assert spm.Smooth._jobtype == 'spatial'
    assert spm.Smooth._jobname == 'smooth'


def test_dartel():
    assert spm.DARTEL._jobtype == 'tools'
    assert spm.DARTEL._jobname == 'dartel'


def test_dartelnorm2mni():
    assert spm.DARTELNorm2MNI._jobtype == 'tools'
    assert spm.DARTELNorm2MNI._jobname == 'dartel'
